#ifndef _CHANNEL3_H_
#define _CHANNEL3_H_

// Kommunikationsbasis einbinden
//#include "Channel2.h"
#include "ChannelBase.h"

#include "../Common/CRC.h"
#include "../Common/AES.h"

// Maximale Laenge eines Fehlertextes
#define ERROR_TEXT_SIZE 255

// MODULE_HARDWARE( 0 )
#define MODULE_HARDWARE      0

// Ping: PING( 0 )
#define PING                 0

// Setzen des Geraetetyp
#define SET_DEVICE_TYPE      (PING + 1)
// (unsigned char)(unsigned short)
// Ermitteln des Geraetetyp
#define GET_DEVICE_TYPE      (SET_DEVICE_TYPE + 1)
// (unsigned char)
// 0 -> PZE Master IV
// 1 -> Zutritt Master IV
// 2 -> BDE Master IV

// Setzen des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
#define SET_BOARD_TYPE                 (GET_DEVICE_TYPE + 1)
// Ermitteln des Boardtyp
#define GET_BOARD_TYPE                 (SET_BOARD_TYPE + 1)

// Setzen der Boardkomponente: RS485 fuer Kommunikation mit PC
#define SET_COMPONENT_RS485_0          (GET_BOARD_TYPE + 1)
// Ermitteln der Boardkomponente
#define GET_COMPONENT_RS485_0          (SET_COMPONENT_RS485_0 + 1)

// Setzen der Boardkomponente: TCP/IP
#define SET_COMPONENT_TCPIP            (GET_COMPONENT_RS485_0 + 1)
// Ermitteln der Boardkomponente
#define GET_COMPONENT_TCPIP            (SET_COMPONENT_TCPIP + 1)

// Setzen der Boardkomponente: Modem MC35i
#define SET_COMPONENT_MODEM_MC35i      (GET_COMPONENT_TCPIP + 1)
// Ermitteln der Boardkomponente
#define GET_COMPONENT_MODEM_MC35i      (SET_COMPONENT_MODEM_MC35i + 1)

// Setzen der Boardkomponente: RS485 fuer Zutrittskontrolle
#define SET_COMPONENT_RS485_1          (GET_COMPONENT_MODEM_MC35i + 1)
// Ermitteln der Boardkomponente
#define GET_COMPONENT_RS485_1          (SET_COMPONENT_RS485_1 + 1)


// MODULE_SYSTEM( 1 )
#define MODULE_SYSTEM        (MODULE_HARDWARE + 1)

// Fehler:
// 1 -> Modul unbekannt
// 2 -> Funktion unbekannt
// 3 -> Zu wenige Parameter
// 4 -> Zu viele Parameter
// 5 -> Zu wenig Daten
// 6 -> Zu viele Daten

// Get: Firmwareversion GET_VERSION( 0 )
#define GET_VERSION          0
// Format: XX.XX.XX??????????????

// Set: SerialNum SET_SERIAL( 1 )
#define SET_SERIAL           (GET_VERSION + 1)
// Get: SerialNum GET_SERIAL( 2 )
#define GET_SERIAL           (SET_SERIAL + 1)
	
// Fehlernummer:
// 1 = Pruefung der Seriennummer - Verschluesselung schlug fehl.

// Set: DateTime (YYYY-MM-DD HH:MM:SS\0) SET_RTC( 2 )
#define SET_RTC              (GET_SERIAL + 1)
// Get: DateTime (YYYY-MM-DD HH:MM:SS\0) GET_RTC( 3 )
#define GET_RTC              (SET_RTC + 1)


// Set: Variable (Systemvariablen GPRS) 
#define SET_SYSTEM_VARIABLE  (GET_RTC + 1)
// (Name\0Value\0)->(Name\0) SET_SYSTEM_VARIABLE ( 4 )

// Get: Variable (Systemvariablen GPRS) 
#define GET_SYSTEM_VARIABLE  (SET_SYSTEM_VARIABLE + 1)
// (Name\0)->(Name\0Value\0) GET_SYSTEM_VARIABLE ( 5 )

// Der Name einer Systemvariablen setzt sich auf folgende Art zusammen:
// MAJOR_NAME.MINOR_NAME
// Fr GPRS ist der Majorname MODEM_MC35i.
// Fehlernummer:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = Lnge des Variablenwertes ist zu lang.


// Set: Relaiszustand 
#define SET_RELAY            (GET_SYSTEM_VARIABLE + 1)
// (Name\0Busnummer\0Relaisnummer\0Status\0Zeit\0)
// ->(Name\0Busnummer\0Relaisnummer\0Status\0Zeit\0) SET_RELAY( 6 )

// Get: Relaiszustand 
#define GET_RELAY            (SET_RELAY + 1)
// (Name\0Busnummer\0Relaisnummer\0)
// ->(Name\0Busnummer\0Relaisnummer\0Status\0Zeit\0) GET_RELAY( 7 )

// Die Namen sind: INTERN, TMR (Trmodule), IOM (IOModule)
// Die Busnummer ist die Busnummer bei 0 beginnend. (Bei INTERN eigentlich immer 0)
// Die Relaisnummer ist die Relaisnummer bei 0 beginnend.
// Die Zeit ist eine Angabe in Sekunden (0 bedeutet eine Dauerschaltung in den gewhlten Status)
// Fehlernummer:
// 1 = Name undefiniert
// 2 = Busnummer undefiniert
// 3 = Relaisnummer undefiniert
// 4 = Status undefiniert  (0, 1) (Knnte irgendwann um andere Varianten erweitert werden)

// Externen Flash schreiben/lesen
#define SET_FLASH            (GET_RELAY + 1)
#define GET_FLASH            (SET_FLASH + 1)

// EEPROM schreiben/lesen
#define SET_EEPROM           (GET_FLASH + 1)
#define GET_EEPROM           (SET_EEPROM + 1)

// Program-Flash schreiben/lesen
#define SET_PROGRAM          (GET_EEPROM + 1) // (wird nur aufgenommen)
#define GET_PROGRAM          (SET_PROGRAM + 1)

#define BEGIN_SEGMENT         (GET_PROGRAM + 1)
// -> [unsigned short(SS_...)unsigned int(address)unsigned int(size)unsigned int(crc)]
// <- [unsigned short(SS_...)unsigned int(address)unsigned int(size)unsigned int(crc)]
#define BEGIN_SEGMENT_OK      (BEGIN_SEGMENT + 1)

#define END_SEGMENT           (BEGIN_SEGMENT_OK + 1)
// -> [unsigned short(SS_...)]
// <- [unsigned short(SS_...)]
// Im Bestaetigungspaket fuer Befehlerhalt ist der CRC = 0 und 
// im Bestaetigungspaket fuer CRC pruefung ist der CRC = dem bei SYSTEM_START gesendeten.
// Ist der CRC falsch, gibt es ein Fehlerpaket mit Fehlernummer 6.

// Die folgenden SS_.. gelten fuer SYSTEM_START und SYSTEM_END
#define SS_UPDATE            0 // Es werden Daten gesetzt die nachdem Uebernommen werden.
#define SS_UPDATE_BG         1 // Es werden Daten gesetzt die nachdem im Hintergrund uebernommen werden.
#define SS_FIRMWARE          2
#define SS_FONTS             3
#define SS_GRAFICS           4
#define SS_SETUP             5
#define SS_LIST              6
// 0 -> Global fuer Uebernahmerahmen von Datenbundel
// 1 -> Firmware
// 2 -> Fonts
// 3 -> Grafik
// 4 -> Setup

// Fehler:
// 1 -> Unbekannter SS-Type
// 2 -> Zu viele Updatesegmente (7 vordefiniert)
// 3 -> Angeforderter Speicherplatz steht nicht zur Verfgung
// 4 -> Kein Stopp vor Start
// 5 -> Verschachtelungsfehler von Stopp und Start. {[]()} -> OK, aber {[( -> Fehler
// 6 -> CRC Berechnung schlug fehl.

#define END_SEGMENT_OK        (END_SEGMENT + 1)
// Wird vom Geraet aufgrund eines SYSTEM_START -> SYSTEM_END mit nachfolgender Berechnung vom CRC gesendet.

// Bootloaderversion abfragen
#define GET_VERSION_BOOTLOADER (END_SEGMENT_OK + 1)
// Format: XX.XX.XX??????????????

// Speicherbereiche reseten
#define SYSTEM_RESET         (GET_VERSION_BOOTLOADER + 1)

// Resetmodus:
// Geraet in einen Zustand versetzen, wie nach dem Bootloader einspielen.
#define SR_ALL               0


// Fehler:
// 1 -> Resetmodus unbekannt.


// Ins Bios wechseln
#define ENTER_BIOS			 (SYSTEM_RESET + 1)
// Bioswechsel erfolgt
#define ENTERED_BIOS		 (ENTER_BIOS + 1)

// Bios verlassen (aenderungen uebernehmen
#define EXIT_BIOS			 (ENTERED_BIOS + 1)
// Biosverlassen erfolgt
#define EXITED_BIOS			 (EXIT_BIOS + 1)

// Setzt/liest das zu verwendende Interface
#define SET_INTERFACE        (EXITED_BIOS + 1)
// 0 -> RS232
// 1 -> TCP/IP
// 2 -> GSM
// 3 -> GPRS
// 4 -> RS485
#define GET_INTERFACE        (SET_INTERFACE + 1)

// Setzt/liest die zu verwendende BaudRate
#define SET_BAUD             (GET_INTERFACE + 1)
#define GET_BAUD             (SET_BAUD + 1)
// Setzt/liest die zu verwendende DeviceID
#define SET_DEVICE_ID        (GET_BAUD + 1)
#define GET_DEVICE_ID        (SET_DEVICE_ID + 1)

// Befehl der beim Geraetestart den Bootloader des ZK - Master IV wiederherstellt.
#define ENTER_BOOTLOADER     (GET_DEVICE_ID + 1)

// In Monitor Mode von XPort wechselt
#define ENTER_XPORT_MONITOR    (ENTER_BOOTLOADER + 1)
// Antwort in Monitor Mode gewechselt
#define ENTER_XPORT_MONITOR_OK (ENTER_XPORT_MONITOR + 1)

// Monitorbefehl an XPort und Antwort erhalten.
#define CMD_XPORT              (ENTER_XPORT_MONITOR_OK + 1)

// In Monitor Mode von XPort wechselt
#define EXIT_XPORT_MONITOR     (CMD_XPORT + 1)
// Antwort in Monitor Mode gewechselt
#define EXIT_XPORT_MONITOR_OK  (EXIT_XPORT_MONITOR + 1)


// MODULE_APPLICATION( 2 )

#define MODULE_APPLICATION   (MODULE_SYSTEM + 1)

// Fehler:
// 1 -> Modul unbekannt
// 2 -> Funktion unbekannt
// 3 -> Zu wenige Parameter
// 4 -> Zu viele Parameter

// Clear: Alle Datensaetze loeschen. CLEAR_RECORDS( 0 )
#define CLEAR_RECORDS        0
// Set: Datensatz quittieren. SET_RECORD( 1 )
#define SET_RECORD           (CLEAR_RECORDS + 1)
// Get: Datensatz hohlen. GET_RECORD( 2 )
#define GET_RECORD           (SET_RECORD + 1)

// Datensatzanfrage
//#define MB_CMD_READ_RECORD              0x40  /* Datensatz senden */

#define MB_ANSW_NO_RECORD               0xC0  /* kein Datensatz vorhanden */
#define MB_ANSW_OFFLINE_RECORD          0xC1  /* Offline Datensatz */
#define MB_ANSW_OLD_OFFLINE_RECORD      0xC3  /* wiederholt gesendeter Offline Datensatz */
#define MB_ANSW_ONLINE_RECORD           0xE1  /* Online Datensatz */
#define MB_ANSW_OLD_ONLINE_RECORD       0xE3  /* wiederholt gesendeter Online Datensatz */

// Datensatzquittierung
//#define MB_CMD_ACK_RECORD               0x48  /* Quittung fuer Datensatz */

#define MB_ANSW_ACK_OFF_RECORD	        0xC8  /* Quittung erhalten (fuer Offline-Datensatz) */
#define MB_ANSW_OLD_ACK_OFF_RECORD      0xCA  /* wiederholt gesendete Quittung */
#define MB_ANSW_ACK_ON_RECORD           0xE8  /* Quittung erhalten (fuer Online-Datensatz) */
#define MB_ANSW_OLD_ACK_ON_RECORD       0xEA  /* wiederholt gesendete Quittung */
#define MB_ANSW_TIMEOUT_ACK_RECORD      0xE9  /* Quittung kam nach Timeout (fuer Online-Datensatz) */
#define MB_ANSW_OLD_TIMEOUT_ACK_RECORD  0xEB  /* wiederholte Quittung nach Timeout */

// Datensatzloeschbefehl
//#define MB_CMD_DEL						0x49  /* Loescht alle Datensaetze. */



// Set: Message SHOW_MESSAGE( 3 )
#define SHOW_MESSAGE          (GET_RECORD + 1)
// (UIN8(Time)unsigned char(Action)unsigned char(Audio)Message\0)
// -> ()
// Time ist die anzuzeigende Dauer
// Action ist die durchzufuehrende Aktion (unbenutzt und immer 0)
// Audio ist das zu verwendende akustische Signal
// Message ist der eigentliche Nachrichtentext

// Fehler:
// 1 -> Nachrichtentext zu lang fuer die Anzeige.
// 2 -> Es wird bereits eine Nachricht angezeigt.


// Set: Variable (Setup) SET_VARIABLE( 4 )
#define SET_VARIABLE         (SHOW_MESSAGE + 1)
// Get: Variable (Setup) GET_VARIABLE( 5 )
#define GET_VARIABLE         (SET_VARIABLE + 1)

// Der Name einer Variablen setzt sich auf folgende Art zusammen:
// MAJOR_NAME.MINOR_NAME
// Der Majorname einer Setupvariablen ist immer SETUP.
// Fehlernummer:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = Lnge des Variablenwertes ist zu lang.


// MODULE_TABLE( 3 )

#define MODULE_TABLE         (MODULE_APPLICATION + 1)

// Fehler:
// 1 -> Modul unbekannt
// 2 -> Funktion unbekannt
// 3 -> Zu wenige Parameter
// 4 -> Zu viele Parameter

// Get: Datensatz-/Listenbeschreibung. GET_TABLE( 0 )
#define GET_TABLE            0
// (Majorname.Minorname\0)
// (STable2)

// Die Tabellenbeschreibung kann auf folgende Arten ermittelt werden.
// Zum einen ist es noetig die Datensatz-/Listenbeschreibungen des Setup sowie der
// Zutrittskontrolle auseinanderzuhalten. Dann wird entweder der Index oder der Name,
// der jeweilingen Tabellendefinition benoetigt.

// Majornamen:
// RECORD - Sind die Datensatzbeschreibungen
// LIST   - Sind die Listenbeschreibungen
// ACCESS - Sind die Listenbeschreibungen der Zutrittskontrolle

// darauf folgt nach einem Punkt entweder die Nummer oder der Name der Tabelle.

// Get: Datensatz-/Listenbeschreibungsfeld. GET_FIELD( 1 )
#define GET_FIELD            (GET_TABLE + 1)
// (Majorname.Minorname.Subminorname\0)
// (STableField)

// Majorname.Minorname bauen sich wie bei den Tabellen auf. Der Subminorname entspricht
// in seinem Aufbau dem Minornamen, ist logischerweise jedoch Feldbezogen.

// Fehler:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = SUBMINOR_NAME ist unbekannt


// Insert RECORD_INSERT(  )
// (Name\0...\0)
// -> (Name\0)

// Delete RECORD_DELETE(  )
// (Name\0KeyFeld\0)
// -> (Name\0)
// Name ist der Tabellenname
// KeyFeld ist der Inhalt der gewnschten Datenzeile

/*
#pragma pack(1)

typedef struct StructTable2 {
	unsigned char num;     // Nummer der Tabelle im System
	char name[32]; // Name der Datensatzbeschreibung
	unsigned char count;   // Anzahl Felder im Datensatz 
	unsigned char key;     // Schluesselspalte
	unsigned char sort;    // Sortierspalte
	unsigned short flags;  // Flags
} STable2;

typedef struct StructTable2Field {
	unsigned char num;       // Nummer des Tabellenfeldes
	char name[32];   // Name des Feldes
	UCHAR type;      // Aufzaehlung Typ des Feldes 
	UCHAR size;      // Laenge des Feldes inc. anbschlieszende '\0' bei Strings 
	unsigned short flags;    // Flags
	char format[32]; // Formatstring fuer Feldwertmoeglichkeit
} STable2Field;

#pragma pack()
*/

#include "Channel2Def.h"

class CChannel3
{
public:
	CChannel3();
	virtual ~CChannel3();

public:
	unsigned char m_data[V_MAX_PAKET - 21]; // 21 Byte Paketframe

	// Zuletzt ausgefuehrte Modul/Funktions
	unsigned short m_lastModule;
	unsigned short m_lastFunction;

	// Verwendeter Returnwert
	int m_result;

	// Fehlertexte fuer die Kommunikation/Module/Funktionen
	unsigned short m_errorCommunicationNum;
	char m_errorCommunication[ERROR_TEXT_SIZE];
	void GetErrorCommunication(unsigned short error, char *text);

	unsigned char m_errorModuleNum;
	char m_errorModule[ERROR_TEXT_SIZE];
	void GetErrorModule(unsigned short module, unsigned char error, char *text);

	unsigned char m_errorFunctionNum;
	char m_errorFunction[ERROR_TEXT_SIZE];
	void GetErrorFunction(unsigned short module, unsigned short function, unsigned char error, char *text);

	void TraceError(unsigned short deviceIDRemote, unsigned short module, unsigned short function);

private:
	unsigned int m_crc32_table[256]; // Lookup table array
	void InitCRC32Table();
	unsigned int Reflect(unsigned int ref, char ch);
public:
	unsigned int GetCRC32(unsigned char *data, unsigned int size, unsigned int crc = 0xFFFFFFFF, unsigned char closure = TRUE);
	unsigned short GetCRC16(unsigned char *data, unsigned int size, unsigned short crc = 0xFFFF);
	void MkPaketCrc(unsigned char *buf,int cnt);
	int CheckPaketCrc(unsigned char *buf, int cnt);

	// Einfache Anfrage ob Geraet mit Protokollversion 3 erreichbar.
	int Request(unsigned short deviceIDRemote);

public:
	void SetChannelBase(CChannelBase& channelBase) { m_channelBase = &channelBase; }
	CChannelBase& GetChannelBase() { return (CChannelBase&)*m_channelBase; }

private:
	unsigned char m_transmit[V_MAX_PAKET];
	unsigned char m_receive[V_MAX_PAKET];
	CChannelBase *m_channelBase;
	short m_deviceID; // Die zu verwendende DeviceID der Klasse
	short m_deviceIDRemote; // Die zu verwendete DeviceID des Remote der Klasse

public:
	short GetDeviceID() { return m_deviceID; }
	short SetDeviceID(short deviceID) { 

		short deviceIDPrev = m_deviceID; 
		m_deviceID = deviceID; 
		return deviceIDPrev; 
	}
	short GetDeviceIDRemote() { return m_deviceIDRemote; }
	short SetDeviceIDRemote(short deviceIDRemote) { 

		short deviceIDPrev = m_deviceIDRemote; 
		m_deviceIDRemote = deviceIDRemote; 
		return deviceIDPrev; 
	}

protected:
	// Verarbeitung eines Befehls
	int Execute(unsigned short deviceIDRemote, unsigned short modul, unsigned short function, unsigned char *buf, unsigned short &count, unsigned char retries = A_VERSUCHE_OMB);
	int Pending(unsigned short deviceIDRemote, unsigned short &modul, unsigned short &function, unsigned char *buf, unsigned short &count, unsigned char retries = A_VERSUCHE_PENDING);

	// Paket versenden
	int Transmit(unsigned char *buf, unsigned short count, int transmitTrials = 3);
	// Paket empfangen
	int Receive(unsigned char *buf, unsigned short &count, int receiveTrials = 3);

public:
	void Reset();
	void ResetError();

	void SetCBFunction(int (*pFunction)(void));
	void SetCBFunctionWithParams(int (*pFunction)(unsigned int, unsigned int));

	int (*m_callBack1)(void);
	int (*m_callBack2)(unsigned int size, unsigned int count);

public:

	// MODULE_HARDWARE

	// Einfache Anfrage ob das Geraet antwortet.
	int Ping(unsigned short deviceIDRemote);

	// Geraetetyp setzen/lesen
	int SetDeviceType(unsigned short deviceIDRemote, unsigned char type);
	int GetDeviceType(unsigned short deviceIDRemote, unsigned char &type);

	// Boardtyp setzen/lesen
	int SetBoardType(unsigned short deviceIDRemote, unsigned char type);
	int GetBoardType(unsigned short deviceIDRemote, unsigned char &type);

	// Komponente setzen/lesen
	int SetComponentRS485_0(unsigned short deviceIDRemote, unsigned char value);
	int GetComponentRS485_0(unsigned short deviceIDRemote, unsigned char &value);

	// Komponente setzen/lesen
	int SetComponentTCPIP(unsigned short deviceIDRemote, unsigned char value);
	int GetComponentTCPIP(unsigned short deviceIDRemote, unsigned char &value);

	// Komponente setzen/lesen
	int SetComponentModemMC35i(unsigned short deviceIDRemote, unsigned char value);
	int GetComponentModemMC35i(unsigned short deviceIDRemote, unsigned char &value);

	// Komponente setzen/lesen
	int SetComponentRS485_1(unsigned short deviceIDRemote, unsigned char value);
	int GetComponentRS485_1(unsigned short deviceIDRemote, unsigned char &value);


	// MODULE_SYSTEM

	// Systemreset; Speicherbereiche loeschen
	int SystemReset(unsigned short deviceIDRemote, unsigned short modus);

	// Firmwareversion lesen
	int GetVersion(unsigned short deviceIDRemote, char *version);
	int GetVersionBootloader(unsigned short deviceIDRemote, char *version);

	// Seriennummer setzen/lesen
	int SetSerial(unsigned short deviceIDRemote, unsigned int serial);
	int GetSerial(unsigned short deviceIDRemote, unsigned int &serial);

	// Datum/Uhrzeit setzen/lesen
	int SetRTC(unsigned short deviceIDRemote, char *time);
	int GetRTC(unsigned short deviceIDRemote, char *time);

	// Systemvariable setzen/lesen
	int SetSystemVariable(unsigned short deviceIDRemote, char *name, char *value);
	int GetSystemVariable(unsigned short deviceIDRemote, char *name, char *value);

	// Relais setzen/lesen
	int SetRelay(unsigned short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time);
	int GetRelay(unsigned short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time);

	// Datensegmentuebertragung beginnen/enden
	int BeginSegment(unsigned short deviceIDRemote, unsigned short art, unsigned int &address, unsigned int size, unsigned int crc);
	int EndSegment(unsigned short deviceIDRemote, unsigned short art, unsigned int crc);

	// Flash setzen/lesen (die ersten 16kByte Bootloader koennen nur gelesen werden)
	int SetFlash(unsigned short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned int count);
	int GetFlash(unsigned short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned int count);

	// EEPROM setzen/lesen
	int SetEEPROM(unsigned short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned short count);
	int GetEEPROM(unsigned short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned short count);

	// PROGRAM setzen/(lesen unterbunden)
	int SetProgram(unsigned short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned short count);
	int GetProgram(unsigned short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned short count);

	// Systembios aufrufen/verlassen
	int EnterBios(unsigned short deviceIDRemote);
	int ExitBios(unsigned short deviceIDRemote);

	// Interface setzen/lesen (RS232, TCPIP...)
	int SetInterface(unsigned short deviceIDRemote, unsigned char type);
	int GetInterface(unsigned short deviceIDRemote, unsigned char &type);
	
	// Baud-Rate setzen/lesen
	int SetBaud(unsigned short deviceIDRemote, unsigned int br);
	int GetBaud(unsigned short deviceIDRemote, unsigned int &br);

	// DeviceID setzen/lesen
	int SetDeviceIDBios(unsigned short deviceIDRemote, unsigned short id);
	int GetDeviceIDBios(unsigned short deviceIDRemote, unsigned short &id);

	// Bootloader aktivieren
	int EnterBootloader(unsigned short deviceIDRemote);

	// Monitor Mode vom XPort
	int EnterXPortMonitor(unsigned short deviceIDRemote, char *data);
	int CommandXPort(unsigned short deviceIDRemote, char *buf, unsigned short &count);
	int ExitXPortMonitor(unsigned short deviceIDRemote);


	// MODULE_APPLICATION

	// Datensaetze loeschen
	int ClearRecords(unsigned short deviceIDRemote);

	// Datensatz quittieren/anfordern
	int SetRecord(unsigned short deviceIDRemote, unsigned char &status);
	int GetRecord(unsigned short deviceIDRemote, unsigned char *buf, unsigned short &count, unsigned char &status);

	// Displaynachicht senden
	int ShowMessage(unsigned short deviceIDRemote, unsigned char time, unsigned char action, unsigned char audio, char *text);

	// Variable setzen/lesen
	int SetVariable(unsigned short deviceIDRemote, char *name, char *value);
	int GetVariable(unsigned short deviceIDRemote, char *name, char *value);


	// MODULE_TABLE

	// Tabellendefinition ueber Index oder Namen ermitteln.
	int GetTable(unsigned short deviceIDRemote, char *name, STable2 &table);
	// Tabellenfelddefinition ueber Index oder Namen ermitteln.
	int GetField(unsigned short deviceIDRemote, char *name, STable2Field &field);
};

#endif
