/**
* @file  TimeboyStation.cpp
* @brief Implementierung der Klasse CTimeboyStation.
*/
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TimeboyStation.h"
#ifdef _WIN32
#else
#include </usr/include/pthread.h>
#endif


// ##################################################################
/**
* @brief Destruktor schliesst die eventuell noch geoeffnete Schnittstelle. 
*
*/
CTimeboyStation::~CTimeboyStation() 
{
}


//////////////////////////////////////////////////////////////////////
// Members
//////////////////////////////////////////////////////////////////////

//#########################################################
//######################### CRC ###########################
//#########################################################

// ##################################################################
/**
* @brief Erzeugt einen 16bit CRC Wert fuer die uebergebenen Daten.
*/
unsigned short CTimeboyStation::GetCRC16(unsigned char *data, unsigned int size, unsigned short crc)
{
	GetChannelBase().Trace("%s(%i):INFO(3): CTimeboyStation::GetCRC16 [ 0x%08X, %i, 0x%04X ]\n", __FILE__, __LINE__, data, size, crc);

	unsigned int i;
	unsigned char j;

	for (i=0; i < size; i++)
	{
		crc = crc ^ data[i];

		for (j=0; j < 8; j++)
		{
			if (crc & 0x1)
			{
				crc >>= 1; 
				crc = crc ^ 0xA001;
			}
			else 
			{
				crc >>= 1;
			}
		}
	}

	return crc;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
unsigned short CTimeboyStation::GetSlotFirmwareVersion(short deviceID)
{
	unsigned short ver;
	int res, autodetect = 1;

	// 254 kann nur Einzeldocking sein und nutzt MasterIV Protokoll.
	// Bereich pruefen
	if (deviceID < 1 || deviceID >= 254)
	{
		return 0;
	}

	switch (m_timeboySlot[deviceID].GetVersion()) 
	{
	case 0xFFFF: // Pruefung bereits durchgefuehrt und war nicht erfolgreich.
		break;

	case 0: 
		{ // Ermitteln der FWVersionProtokollversion
			CChannelBase::enumType type;
			char commString[256];
			int commVal, commTO;
			int commValOld = 0;
			GetChannelBase().GetOpenParams(type, commString, commVal, commTO);

			while (autodetect)
			{
				// Bei Seriell ist es wahrscheinlicher, dass mit 9600bd kommuniziert wird, deswegen zuerst damit Pruefen und dann mit 38400bd.
				if (type == CChannelBase::Serial && commVal != 9600)
				{
					// Schnittstellenbaud anpassen
					commValOld = commVal;
					commVal = 9600;
					GetChannelBase().ChangeStateSerial(GetChannelBase().m_comm, 9600, commTO);
				}
				else 
				{
					// Wurde Baud per autodetect abgeaendert?
					if (commValOld)
					{
						GetChannelBase().ChangeStateSerial(GetChannelBase().m_comm, commValOld, commTO);
					}

					autodetect = 0;
				}

				// In der Version 1.25 (08.2007) wurde die Timeboyerkennung per Kommunikationspolling der Docking auf TB vorgenommen.
				// Ab 1.26 (09.2007) wurde die Untersttzung der Timeboyerkennung per Komperatorschaltung integriert.
				// De-/Aktivierung ber 0x8000000C (Wert 2 kann dort verwendet werden), abfrage des Zustandes ber 0x8000E000.

				if ((res = ReadBytes(deviceID, MB_SLOT_VERSION, 0, (unsigned char*)&ver, sizeof(ver))) == IV_ERROR_SUCCESS)
				{
					// < 1.31	Paketpuffer ist 45 Byte
					// >= 1.31	Paketpuffer ist 150 Byte
					m_timeboySlot[0].SetVersion(commVal); // Im 0 Byte die verwendete Baudrate halten.
					m_timeboySlot[deviceID].SetVersion( BinToUIntMSBFirst((unsigned char*)&ver) );
					return m_timeboySlot[deviceID].GetVersion();
				}
				else
				{
					m_timeboySlot[deviceID].SetVersion( 0xFFFF );
				}
			}
			break;
		}

	default:
		return m_timeboySlot[deviceID].GetVersion();
	}

	return 0;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void CTimeboyStation::ResetSlot(short deviceID)
{
	if (deviceID < 0 || deviceID >= 254)
	{
		return;
	}

	m_timeboySlot[deviceID].Reset();
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void CTimeboyStation::ResetSlots()
{
  unsigned char deviceID;
  for (deviceID = 0; deviceID < 255; deviceID++) {
    m_timeboySlot[deviceID].Reset();
  }
}

int CTimeboyStation::ExecuteDocking(unsigned char *txBuf, unsigned char txCount, unsigned char *rxBuf, unsigned char *rxCount)
{
	int res, nb;
	int rcnt=0,len;

	// Autobaud
	// Stimmt die aktuell Verwendete Baudrate nicht mit der benoetigten ueberein?
	if (GetChannelBase().GetType() == CChannelBase::Serial && m_timeboySlot[0].GetVersion() && GetChannelBase().GetBaud() != m_timeboySlot[0].GetVersion())
	{
		GetChannelBase().ChangeStateSerial(GetChannelBase().m_comm, m_timeboySlot[0].GetVersion(), GetChannelBase().GetTimeout());
	}

	// CRC noch an Paket anfuegen
	*(unsigned short *)&txBuf[txCount] = GetCRC16(txBuf, txCount);
	txCount += sizeof(unsigned short);

	// Empfangsanzahl initialisieren
	*rxCount = 0;

	GetChannelBase().Trace("%s(%i):INFO(1): CChannel4Protocol::ExecuteDocking.send( %i ) ( %s[ 0x%02X ], Add: 0x%08X  Cnt: %i )\n", __FILE__, __LINE__, txBuf[0], (txBuf[3] == MB_READ) ? "READ" : "WRITE", txBuf[3], BinToULongMSBFirst(&txBuf[4]), (txBuf[3] == MB_READ) ? txBuf[9] : txBuf[2] - 0x0A);

	// Paketdaten versenden.
	if ((res = GetChannelBase().TransmitData((void*)txBuf, txCount, nb, 1)) != IV_ERROR_SUCCESS)
	{
		GetChannelBase().Trace("%s(%i):ERROR(1): CChannel4Protocol::ExecuteDocking.transmit [ TransmitData fail with: %i ]\n", __FILE__, __LINE__, res);
		// Neuerkennung notwendig.
		ResetSlot(GetDeviceIDRemote());
		return res;
	}
	// Alle Zeichen versendet?
	if (nb != (unsigned)txCount)
	{
		return -IV_ERROR_TRANSMIT;
	}

	// Kopf des Paketes empfangen.
	if ((res = GetChannelBase().ReceiveData((void*)rxBuf, 4, nb, 1)) != IV_ERROR_SUCCESS)
	{
		GetChannelBase().Trace("%s(%i):ERROR(1): CChannel4Protocol::ExecuteDocking [ ReceiveData fail with: %i ]\n", __FILE__, __LINE__, res);
		// Neuerkennung notwendig.
		ResetSlot(GetDeviceIDRemote());
		return res;
	}
	// Alle Zeichen empfangen?
	if (nb < 4) 
	{
		GetChannelBase().Trace("%s(%i):ERROR(1): CChannel4Protocol::ExecuteDocking [ %i von %i Byte empfangen ]\n", __FILE__, __LINE__, nb, 4);
		return -IV_ERROR_RECEIVE;
	}
	rcnt = nb;

	// Busadresse vergleichen
	if (txBuf[0] != rxBuf[0])
	{
		//#########################
		// Bugfix fuer Dockingstation FW Version 1.24
		// Naechstes Byte ist Adresse?
		if (txBuf[0] == rxBuf[1])
		{
			//GetChannelBase().Trace("%s(%i):ERROR(1): CChannel4Protocol::ExecuteDocking [ Skip wrong start byte {0x%02X} ]\n", __FILE__, __LINE__, rxBuf[0]);
			rxBuf[0] = rxBuf[1];
			rxBuf[1] = rxBuf[2];
			rxBuf[2] = rxBuf[3];
			rcnt--;
		}
		else
		{
			// Delay bei TCP/IP ueber Com-Server
			if (GetChannelBase().GetType() == CChannelBase::Socket)
			{
				Sleep( 200 );
			}

			GetChannelBase().Trace("%s(%i):ERROR(1): CChannel4Protocol::ExecuteDocking [ address wrong %i != %i ]\n", __FILE__, __LINE__, txBuf[0], rxBuf[0]);
#ifdef _DEBUG
			int i; 
			for (i=0; i < nb; i++) 
			{
				GetChannelBase().Trace("%02X ", rxBuf[i]);
			}
			GetChannelBase().Trace("\n");
#endif
			GetChannelBase().Flush();
			return -IV_ERROR_ADDRESS;
		}
	}

	// Solllaenge Paket aus Kopf ermitteln
	len = BinToUIntMSBFirst(&rxBuf[1]);
	if (len < 7 || len > 200) 
	{
		// Delay bei TCP/IP ueber Com-Server
		if (GetChannelBase().GetType() == CChannelBase::Socket)
		{
			Sleep( 200 );
		}

		GetChannelBase().Trace("%s(%i):ERROR(1): CChannel4Protocol::ExecuteDocking [ packet length < 7 || > 200 {%i}]\n", __FILE__, __LINE__, len);
#ifdef _DEBUG
		int i; 
		for (i=0; i < nb; i++) 
		{
			GetChannelBase().Trace("%02X ", rxBuf[i]);
		}
		GetChannelBase().Trace("\n");
#endif
		GetChannelBase().Flush();
		return -IV_ERROR_PACKETLENGTH;
	}

	// Body des Paketes empfangen.
	if ((res = GetChannelBase().ReceiveData((void*)&rxBuf[rcnt], len-rcnt, nb, 1)) != IV_ERROR_SUCCESS) 
	{
		GetChannelBase().Trace("%s(%i):ERROR(1): CChannel4Protocol::ExecuteDocking [ ReceiveData fail with: %i ]\n", __FILE__, __LINE__, res);
		// Neuerkennung notwendig.
		ResetSlot(GetDeviceIDRemote());
		return res;
	}
	// Alle Zeichen empfangen?
	if (nb < (len-rcnt)) 
	{
		GetChannelBase().Trace("%s(%i):ERROR(1): CChannel4Protocol::ExecuteDocking [ Nachlesen %i von %i Byte empfangen ]\n", __FILE__, __LINE__, nb, (len-rcnt));
		GetChannelBase().Flush();
		return -IV_ERROR_RECEIVE;
	}
	rcnt += nb;

	// Alle Zeichen empfangen?
	if (GetCRC16(rxBuf, rcnt)) 
	{
		GetChannelBase().Trace("%s(%i):ERROR(1): CChannel4Protocol::ExecuteDockinge [ Paket CRC ]\n", __FILE__, __LINE__);
		return -IV_ERROR_CRC_PACKET;
	}

	// Handelt es sich um eine Steuerantwort der Dockingstation?
	if (rcnt == 7)
	{ 
		GetChannelBase().Trace("%s(%i):INFO(1): CChannel4Protocol::ExecuteDocking.receive ( Befehl: 0x%02X, Fehler: 0x%02X )\n", __FILE__, __LINE__, rxBuf[3], rxBuf[4]);
	}
	else
	{
		GetChannelBase().Trace("%s(%i):INFO(1): CChannel4Protocol::ExecuteDocking.receive ( Befehl: 0x%02X, Lnge: %i )\n", __FILE__, __LINE__, rxBuf[3], rcnt);
	}

	// Zeitstempel der Antwort halten
	m_timeboySlot[GetDeviceIDRemote()].SetReceiveStampSlot( clock() );

	// Anzahl empfangener Bytes
	*rxCount = rcnt;

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
* @brief Prueft die Erreichbarkeit des Kommunikationspartners.
*
* @param deviceID Anzusprechender Kommunikationspartner.
* @param model Erreichtes Timeboymodel.
*/
int CTimeboyStation::GetTimeboyModel(short deviceID, int &model)
{
	GetChannelBase().Trace("%s(%i):INFO(3): CChannel4Protocol::GetTimeboyModel [ %i ]\n", __FILE__, __LINE__, deviceID);
	int res = IV_ERROR_SUCCESS;
	unsigned char data[2];
	memset(data, 0, sizeof(data));

	// Ist die schnelle Steck-Erkennung verfuegbar?
	if (m_timeboySlot[deviceID].GetDetectionState() == 2)
	{
		// Pruefung ob der Slot auch meldet, dass ein Timeboy steckt.
		if ((res = ReadBytes(deviceID, MB_SLOT_STATE, 0, data, 2, 1)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			return res;
		}
		if (*(unsigned short*)data != 0x0100)
		{
			// Timeboy ist nicht erreichbar, da nicht gesteckt!
//			return -IV_ERROR_UNREACHABLE;
      // KORREKTUR (2012_05_14):
      // Da ab der TimeboyIV Version 04.02.02.40 die Ladeabschaltung voll unterstuetzt wird,
      // Kommt es durch die Abfrageroutine zu Fehlinformationen, da die Ladeschaltung meldet,
      // das kein Timeboy steckt obwohl dieses der Fall ist.
		}
	}

	do 
	{
		// Timeboy ansprechen.
		if ((res = ReadBytes(deviceID, 0, 0, data, 2, 1)) != IV_ERROR_SUCCESS)
		{
			// Wurde ermittelt, dass ein Timeboy steckt? -> Nochmal versuchen.
			if (*(unsigned short*)data == 0x0100)
			{
				*(unsigned short*)data = 0;
				continue;
			}

			// Fehler ausgeben
			return res;
		}
	} while (res != IV_ERROR_SUCCESS);

	if (data[0] == '~' && data[1] == '~')
	{
		model = 4;
	}
	else
	{
		model = 3;
	}

	// Unabhaengig vom Timeboymodel
	// Pruefung auf die Verfuegbarkeit der Timeboyerkennung durchfuehren.
	if (m_timeboySlot[deviceID].GetVersion() >= 0x0126 && m_timeboySlot[deviceID].GetDetectionState() == 0)
	{
		if ((res = ReadBytes(deviceID, MB_SLOT_CHECK, 0, data, 2, 1)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			return res;
		}

		if (*(unsigned short*)data == 0x0002)
		{
			// Wird unterstuetzt
			m_timeboySlot[deviceID].SetDetectionState( 2 );
		}
		else if (*(unsigned short*)data == 0x0001)
		{
			// Pruefung ob der Slot auch meldet, dass ein Timeboy steckt.
			if ((res = ReadBytes(deviceID, MB_SLOT_STATE, 0, data, 2, 1)) != IV_ERROR_SUCCESS)
			{
				// Fehler ausgeben
				return res;
			}

			if (*(unsigned short*)data == 0x0100)
			{
				// Timeboy wurde erkannt und der Slot meldet, dass ein Timeboy steckt -> Komperatorschaltung bestueckt.
				*(unsigned short*)data = 0x0002;
				//if ((m_result = WriteBytes(deviceID, MB_SLOT_CHECK, data, 2, 1)) != IV_ERROR_SUCCESS)
				//{
				//	// Fehler ausgeben
				//	return m_result;
				//}

				m_timeboySlot[deviceID].SetDetectionState( 2 );
			}
			else
			{
				m_timeboySlot[deviceID].SetDetectionState( 1 );
			}
		}
		else
		{
			m_timeboySlot[deviceID].SetDetectionState( 1 );
		}
	}

	return res;
}

// ##################################################################
/**
* @brief Setzt den Zustand der Roten LED der Mehrfachdocking.
*
* @param deviceID Anzusprechender Kommunikationspartner.
*/
int CTimeboyStation::SetDockingLED(short deviceID, int state)
{
	GetChannelBase().Trace("%s(%i):INFO(3): CChannel4Protocol::SetDockingLED [ %i ]\n", __FILE__, __LINE__, deviceID);
	int res = IV_ERROR_SUCCESS;
	unsigned char data[1];

	if (state != 0)
	{
		data[0] = 0x00;
	}
	else
	{
		data[0] = 0xFF;
	}

	if ((res = WriteBytes(deviceID, MB_SLOT_LED, 0, data, 1)) != IV_ERROR_SUCCESS)
	{
		// Fehler ausgeben
		return res;
	}

	GetChannelBase().Trace("%s(%i):INFO(1): CChannel4Protocol::SetDockingLED [ Letzter Wechsel vor %0.3f Sekunden ]\n", __FILE__, __LINE__, float(clock() - m_timeboySlot[deviceID].GetLEDStamp()) / float(CLOCKS_PER_SEC));

	// Zustand der LED und Zeitpunkt des wechsels halten
	m_timeboySlot[deviceID].SetLEDState((state != 0) ? 0x01 : 0xFF);
	m_timeboySlot[deviceID].SetLEDStamp( clock() );

	return res;
}

// ##################################################################
/**
* @brief Setzt den Kommunikationszustand eines Slots der Mehrfachdocking.
*
* @param deviceID Anzusprechender Kommunikationspartner.
* @param state Zu setzender Kommunikationszustand.
*/
int CTimeboyStation::SetTimeboySlotState(short deviceID, int state, int commandResult)
{
	GetChannelBase().Trace("%s(%i):INFO(3): CChannel4Protocol::SetTimeboySlotState [ %i, %i ]\n", __FILE__, __LINE__, deviceID, state);

	if ( IsSlot(deviceID) )
	{
		// Ist der geforderte Zustand gleich dem aktuellen?
		if (m_timeboySlot[deviceID].GetLEDState() == state)
		{
			return IV_ERROR_SUCCESS;
		}

		if (state != 0)
		{
			int model = 0;
			// Es muss ein gueltiges Timeboymodel vorliegen
			if (GetTimeboyModel(deviceID, model) != IV_ERROR_SUCCESS)
			{
				return -IV_ERROR_UNREACHABLE;
			}
			if (model != 4)
			{
				return -IV_ERROR_WRONG_TIMEBOY_MODEL;
			}
			// Rote LED einschalten
			if(SetDockingLED(deviceID, 1) != IV_ERROR_SUCCESS)
			{
				return -IV_ERROR_UNREACHABLE;
			}
		}
		else
		{
			// Rote LED ausschalten
			if(SetDockingLED(deviceID, 0) != IV_ERROR_SUCCESS)
			{
				return -IV_ERROR_UNREACHABLE;
			}
		}
	}
	else
	{
		// Bei der angesprochenen ID handelt es sich um keinen Slot einer TimeboyStation.
		return -IV_ERROR_SLOT_NONE;
	}

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
* @brief Entgegennehmen der Verbindung und starten eines Verarbeitungsthreads.
*/
int CTimeboyStation::StartCommunicationThread() 
{
	GetChannelBase().Trace("%s(%i):INFO(3): CTimeboyStation::StartCommunicationThread [  ]\n", __FILE__, __LINE__);

	// Hauptthread starten
	if (m_communicationThread == 0)
	{	
#ifdef _WIN32
		m_communicationThread = ::CreateThread(NULL, 0, CommunicationThread, this, 0, &m_threadID);
#else
		pthread_create(&m_communicationThread, NULL, CommunicationThread, this);
#endif

		int i;
		// Hochlauf des Workers pruefen
		for (i=0; (m_communicationThread != 0) && (GetThreadState() != idle) && (i < 60); i++)
		{
			Sleep(100);
		}
	}

	return (m_communicationThread != 0);
}

// ##################################################################
/**
* @brief Beendigungssignal im Workerthread setzen und auf dessen beendigung warten.
*/
void CTimeboyStation::StopCommunicationThread()
{
	GetChannelBase().Trace("%s(%i):INFO(3): CTimeboyStation::StopCommunicationThread\n", __FILE__, __LINE__);

	// Socket-Thread deaktivieren, wenn aktiv
	if (m_communicationThread != 0)
	{
		// WorkerThread fuer Socket beenden.
		SetThreadCommand( exit );

		// Thread umschalten
		Sleep(10);

		// Thread Beendigung abwarten (CommunicationThread stellt m_endSignal auf FALSE)
		int i;
		for (i=0; i < 60 && GetThreadState() != end; i++)
		{				
			GetChannelBase().Trace("%s(%i):INFO(1): CTimeboyStation::StopCommunicationThread [ End wait %i ]\n", __FILE__, __LINE__, i+1);
			Sleep(100);
		}

		// Hartes Schliessen, wenn noch nicht selbst beendet.
		if (m_communicationThread)
		{
#ifdef _WIN32
			::CloseHandle(m_communicationThread);
#else
			pthread_exit(&m_communicationThread);
#endif
			m_communicationThread = NULL;
		}
	}
}

// ##################################################################
/**
* @brief Workerthread fuer die Verarbeitung der Kommunikationsaufgaben.
*/
#ifdef _WIN32
unsigned long WINAPI CTimeboyStation::CommunicationThread(LPVOID lpParam)
#else
void *CTimeboyStation::CommunicationThread(void *lpParam)
#endif
{
	if (lpParam == NULL)
	{	
		// Fehlerhafter Parameter
		return 0;
	}

	// Es wird davon ausgegangen, dass der Parameter ein CChannel4 - Objekt ist.
	CTimeboyStation &own = *(CTimeboyStation*)lpParam;

	own.GetChannelBase().Trace("%s(%i):INFO(3): CTimeboyStation::CommunicationThread [ Begin 0x08X ]\n", __FILE__, __LINE__, lpParam);

	// Threadzustand setzen
	own.m_threadState = begin;
	own.m_threadCommand = none;

	// Hauptschleife ausfuehren?
	int run = TRUE;
	int failCount = 0, resumeCount = 0;
	unsigned char deviceID = 1;

	// Hauptschleife des Worker
	while ( run )
	{
		switch ( own.m_threadState )
		{
		case begin:
			// Threadzustand setzen
			own.m_threadState = idle;
			break;

		case idle: {
			if (own.m_threadCommand != none)
			{
				// Threadzustand setzen
				own.m_threadState = executeCommand;
			}
			else
			{
				//// Pruefung ob LED augeschaltet werden muss.
				//if (own.IsFlag(TSF_AUTO_LED))
				//{
				//	for (deviceID=1; deviceID < 255; deviceID++)
				//	{
				//		if (own.m_timeboySlot[deviceID].GetLEDState() && ((clock() - own.m_timeboySlot[deviceID].GetLEDStamp()) > 3))
				//		{
				//			own.SetDockingLED(deviceID, 0);
				//		}
				//	}
				//}

				Sleep(10);
			}
			break;}

		case executeCommand: {

			switch (own.m_threadCommand)
			{
			case exit:
				// Hauptschleife verlassen
				run = FALSE;
				break;

			case write:
				own.m_commandParam.res = own.ThreadWriteBytes(own.m_commandParam.deviceID, own.m_commandParam.baseAddress, own.m_commandParam.offset, own.m_commandParam.buf, own.m_commandParam.count, own.m_commandParam.trials);
				break;

			case read:
				own.m_commandParam.res = own.ThreadReadBytes(own.m_commandParam.deviceID, own.m_commandParam.baseAddress, own.m_commandParam.offset, own.m_commandParam.buf, own.m_commandParam.count, own.m_commandParam.trials);
				break;
			}

			// Aufgabe abgearbeitet
			own.m_threadState = idle;
			own.m_threadCommand = none;
			break;}
		}
	}

	// Thread wird beendet
	own.m_communicationThread = NULL;
	// Threadzustand setzen
	own.m_threadState = end;

	own.GetChannelBase().Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ End ]\n", __FILE__, __LINE__);

#ifndef _WIN32
	pthread_exit(NULL);
#endif
	return 0;
}

//#########################################################
//################## Members ##############################
//#########################################################

/*############################################################################*/
//  FUNKTION:    WriteBytes
/** Schreibbefehl, um Bytes in den Timeboy zu schreiben.

Autor                 : Michael Wicher            @n
erstellt am           : 02.03.2007                @n

@param[in] address Zieladresse im Timeboy.
@param[in] buf Zeiger auf Daten, die gesendet werden sollen.
@param[in] count Anzahl Bytes zum Senden.
@param[in] slot Steckplatz der Docking [1..10].

@return #TRUE -> Erfolg, #FALSE -> Miszerfolg.

@b Aufgaben:
- Bytes an den Timeboy senden.

@b Aenderungen:
- 17.01.2008 (MW) Neuer Transferbefehl, um die Kommunikation durch einen Mutex
zu schtzen.
- 01.02.2008 (MW) Bei allen Befehlen fr den Timeboy wird zuvor bei der Docking
angefragt, ob auch ein Timeboy steckt.
Weiterhin wird hier geprft, ob sich der Task terminieren soll.
Falls kein Timeboy steckt, wird hier auch die Startphase
abgeschloszen, so dasz die Funktion #DOCKINGControl beendet wird.
- 27.03.2008 (MW) Max. fnf Versuche, den Befehl abzusetzen. Dazwischen wird
immer geprft, ob der Timeboy laut Docking steckt.
- 19.05.2008 (MW) Vor Abarbeitung des Befehls wird geprft, ob der ausgewhlte
Steckplatz der Docking noch erreichbar ist.
Bei Befehlen dirket fr die Docking werden nun 2 Versuche
durchgefhrt.
Die CRC-Berechnung verschoben.
*/
/*############################################################################*/

int CTimeboyStation::WriteBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int transmitTrials)
{
	int res = IV_ERROR_SUCCESS;

	// Ueber den Kommunikationsthread kommunizieren?
	if (m_communicationThread)
	{
		m_commandParam.deviceID = deviceID;
		m_commandParam.baseAddress = baseAddress;
		m_commandParam.offset = offset;
		m_commandParam.buf = buf;
		m_commandParam.count = count;
		m_commandParam.trials = transmitTrials;

		// Schreibvorgang starten.
		if (SetThreadCommand( write ))
		{
			while (GetThreadCommand() != none)
			{
				Sleep(10);
			}
		}
		else
		{
			return -IV_ERROR_MODULE_BUSY;
		}

		return m_commandParam.res;
	}

	// Direkte Befehlsausfuehrung
	res = ThreadWriteBytes(deviceID, baseAddress, offset, buf, count, transmitTrials);

	if (res == IV_ERROR_SUCCESS)
	{
		// Nach erfolgreicher Kommunikation automatisch den
		// Kommunikationsthread starten, wenn noch nicht geschehen.
//		StartCommunicationThread();
	}

	return res;
}

int CTimeboyStation::ThreadWriteBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int transmitTrials)
{
	static unsigned char pre[MB_SLOT_BUFFER_SIZE_LARGE];

	int res = IV_ERROR_SUCCESS;
	unsigned char current[MB_SLOT_BUFFER_SIZE_LARGE];
	unsigned char i;
	unsigned char limit;
	unsigned char transmit[MB_SLOT_BUFFER_SIZE_LARGE];
	unsigned short pause;
	unsigned char receive[MB_SLOT_BUFFER_SIZE_LARGE];
	unsigned char repeat;
	unsigned char rxCount;
	unsigned char sendPre;
	unsigned char check = FALSE;
	int executeFails = 0;
	unsigned int address = baseAddress + offset;

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceID);

	unsigned char bufSize = GetDataBufferSize(GetDeviceIDRemote());

	// prfe, ob die Busnummer verstellt ist
	//DOCKINGBusnumVerify(slot);

	for (; count;) 
	{
		sendPre = FALSE;

		for (limit=20; limit; limit >>= 1) 
		{
			// bei allen Befehlen fr den Timeboy bei der Docking anfragen, dasz auch ein Timeboy steckt
			//if ((((UINT8 *)&address)[3] & 0xF0) == 0x40) {

			//	KERNELCheckForTerminate(DOCKINGStop);

			//	// falls kein Timeboy steckt, macht der weitere Ablauf keinen Sinn und man kann abbrechen
			//	if (DOCKINGCheckTimeboy(slot) != DOCKING_TIMEBOY_PUSHED) {

			//		// Startphase abgeschloszen, jetzt darf auch der IO-Task loslaufen
			//		globalDOCKING.ready = TRUE;

			//		return (FALSE);

			//	}

			//} else {

			//	limit = 1;

			//}

			current[0] = (unsigned char)GetDeviceIDRemote();
			current[1] = 0x00;
			current[2] = 0x0A + ((count > bufSize) ? bufSize : count);
			current[3] = 0x22;
			current[4] = ((unsigned char *)&address)[3];
			current[5] = ((unsigned char *)&address)[2];
			current[6] = ((unsigned char *)&address)[1];
			current[7] = ((unsigned char *)&address)[0];

			memcpy((unsigned char *)&current[8], buf, (count > bufSize) ? bufSize : count);

			repeat = limit;

			// je nach Timeboy oder Docking als Empfnger
			if (current[4] == 0x80) 
			{
				// beim Stellen der Busnummer nur ein Versuch
				if ((current[6] == 0x00) && (current[7] == 0x00)) 
				{
					repeat = 1;
					pause = 0;

					// bei allen anderen Anfragen an die Docking zwei Versuche!
				} 
				else 
				{
					repeat = 2;
					pause = 80;
				}
			} 
			else 
			{
				pause = 2000 / repeat;
			}

			for (i=0; i < repeat; i++) 
			{
				//if (i) DEBUGString(TRACE_DEVICE_DOCKING, "repeat write <%u>.", i);

				if (sendPre == TRUE) 
				{
					// lade, falls ntig,  den Vorgnger
					memcpy(transmit, pre, sizeof(transmit));
				} 
				else 
				{
					// aktuelles Paket laden
					memcpy(transmit, current, sizeof(transmit));
				}

				if ((res = ExecuteDocking(transmit, transmit[2] - 2, receive, &rxCount)) == IV_ERROR_SUCCESS) 
				{
					// Anzahl fehlversuche zurcksetzen
					executeFails = 0;

					// Paket fr Docking?
					if (transmit[4] == 0x80) 
					{
						if (*(unsigned int *)&receive[4] == *(unsigned int *)&transmit[4]) 
						{
							break;
						}
					} 
					else // Paket fuer Timeboy
					{
						if (rxCount == 7) 
						{
							//DEBUGString(TRACE_DEVICE_DOCKING, "%02X, %08lX", receive[4], CONVSwapLong(*(UINT32 *)&transmit[4]));

							if (receive[4] == 0x22) 
							{
								//KERNELMilliSleep(100);
								Sleep(100);

								// falls nicht der Vorgnger gesendet wurde
								if (sendPre == FALSE) 
								{
									// gesendetes Paket zur evtl. Wiederholung merken
									memcpy(pre, transmit, sizeof(pre));

									// fertig
									break;
								}

								// Limit neu festlegen
								repeat = limit;

								// jetzt das aktuelle Paket nochmal senden
								sendPre = FALSE;
							} 
							else if ((receive[4] == 0x21) || (receive[4] == 0x24)) 
							{
								//DEBUGString(TRACE_DEVICE_DOCKING, "repeat pre.");

								// falls eine Wiederholung begonnen wird
								if (sendPre == FALSE) 
								{
									// lege Limit neu fest
									repeat = limit;
								}

								sendPre = TRUE;
							}
						}
						else
						{
							//if (*(unsigned int *)&receive[4] == *(unsigned int *)&transmit[4]) 
							//{
							//	break;
							//}
						}
					}
				}
				else
				{
					// Fehlerhafte Ausfuehrungen halten
					executeFails++;
					if (executeFails >= transmitTrials)
					{
						return res;
					}
				}

				if (pause) 
				{
					Sleep(pause);
				}
			}

			// bei erfolgreichem Senden keine Wiederholung
			if (i < repeat) 
			{
				break;
			}
		}

		// falls das Limit erreicht wurde, konnte der Befehl nicht erfolgreich gesendet werden!
		if (limit == 0) 
		{
			return -IV_ERROR_UNREACHABLE;
		}

		address += bufSize;
		buf += bufSize;
		count -= (count > bufSize ? bufSize : count);
	}

	// Zeitpunkt der letzten Timeboykommunikation halten
	if ((address & 0x80000000) == 0)
	{
		m_timeboySlot[GetDeviceIDRemote()].SetReceiveStampTimeboy(clock());
	}

	return res;
}

/*############################################################################*/
//  FUNKTION:    ReadBytes
/** Lesebefehl, um Bytes aus dem Timeboy zu lesen.

Autor                 : Michael Wicher            @n
erstellt am           : 02.03.2007                @n

@param[in] buf Zeiger, wo gelesene Daten hinkopiert werden sollen.
@param[in] address Quelladresse im Timeboy.
@param[in] count Anzahl Bytes zum Lesen.
@param[in] slot Steckplatz der Docking [1..10].

@return #TRUE -> Erfolg, #FALSE -> Miszerfolg.

@b Aufgaben:
- Bytes vom Timeboy lesen.

@b Aenderungen:
- 17.01.2008 (MW) Neuer Transferbefehl, um die Kommunikation durch einen Mutex
zu schtzen.
- 01.02.2008 (MW) Bei allen Befehlen fr den Timeboy wird zuvor bei der Docking
angefragt, ob auch ein Timeboy steckt.
Weiterhin wird hier geprft, ob sich der Task terminieren soll.
Falls kein Timeboy steckt, wird hier auch die Startphase
abgeschloszen, so dasz die Funktion #DOCKINGControl beendet wird.
- 27.03.2008 (MW) Max. fnf Versuche, den Befehl abzusetzen. Dazwischen wird
immer geprft, ob der Timeboy laut Docking steckt.
- 19.05.2008 (MW) Vor Abarbeitung des Befehls wird geprft, ob der ausgewhlte
Steckplatz der Docking noch erreichbar ist.
Bei Befehlen dirket fr die Docking werden nun 2 Versuche
durchgefhrt.
Die CRC-Berechnung verschoben.
*/
/*############################################################################*/

int CTimeboyStation::ReadBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int receiveTrials)
{
	int res = IV_ERROR_SUCCESS;
	// Ueber den Kommunikationsthread kommunizieren?
	if (m_communicationThread)
	{
		m_commandParam.deviceID = deviceID;
		m_commandParam.baseAddress = baseAddress;
		m_commandParam.offset = offset;
		m_commandParam.buf = buf;
		m_commandParam.count = count;
		m_commandParam.trials = receiveTrials;

		// Lesevorgang starten.
		if (SetThreadCommand( read ))
		{
			while (GetThreadCommand() != none)
			{
				//GetChannelBase().Trace("%s(%i):INFO(1): CTimeboyStation::ReadBytes [ wait ]\n", __FILE__, __LINE__);
				Sleep(10);
			}
		}
		else
		{
			return -IV_ERROR_MODULE_BUSY;
		}

		return m_commandParam.res;
	}

	// Direkte Befehlsausfuehrung
	res = ThreadReadBytes(deviceID, baseAddress, offset, buf, count, receiveTrials);

	if (res == IV_ERROR_SUCCESS)
	{
		// Nach erfolgreicher Kommunikation automatisch den
		// Kommunikationsthread starten, wenn noch nicht geschehen.
//		StartCommunicationThread();
	}

	return res;
}

int CTimeboyStation::ThreadReadBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int receiveTrials)
{
	int res = IV_ERROR_SUCCESS;
	unsigned char i;
	unsigned char limit;
	unsigned char transmit[MB_SLOT_BUFFER_SIZE_LARGE];
	unsigned short pause;
	unsigned char receive[MB_SLOT_BUFFER_SIZE_LARGE];
	unsigned char repeat;
	unsigned char rxCount;
	int executeFails = 0;
	int error21Cnt = 0;
	unsigned int address = baseAddress + offset;

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceID);

	unsigned char bufSize = GetDataBufferSize(GetDeviceIDRemote());

	if (offset == MB_SLOT_BUFFER_DATA_SIZE_SMALL)
	{
		address = baseAddress;
		if (bufSize == MB_SLOT_BUFFER_DATA_SIZE_SMALL || (count - MB_SLOT_BUFFER_DATA_SIZE_SMALL) <= MB_SLOT_BUFFER_DATA_SIZE_SMALL)
		{
			// Nur wenn der moegliche Lesepuffer groesser ist, macht es sinn die bereits gelesenen Daten nochmals mitzulesen?
			// Die Dockingstation liest ein Packet vor und sendet immer die zuletzt angeforderte Packetgroesse in Folge.
			address += MB_SLOT_BUFFER_DATA_SIZE_SMALL;
			// bereits gelesene Daten von der geforderten Groesse abziehen
			count -= MB_SLOT_BUFFER_DATA_SIZE_SMALL;
			// Puffer weitersetzen
			buf += MB_SLOT_BUFFER_DATA_SIZE_SMALL;
		}
	}

	// prfe, ob die Busnummer verstellt ist
	//DOCKINGBusnumVerify(slot);

	for (; count;) 
	{
		for (limit=20; limit; limit >>= 1) 
		{
			// bei allen Befehlen fr den Timeboy bei der Docking anfragen, dasz auch ein Timeboy steckt
			//if ((((UINT8 *)&address)[3] & 0xF0) == 0x40) {

			//	KERNELCheckForTerminate(DOCKINGStop);

			//	// falls kein Timeboy steckt, macht der weitere Ablauf keinen Sinn und man kann abbrechen
			//	if (DOCKINGCheckTimeboy(slot) != DOCKING_TIMEBOY_PUSHED) {

			//		// Startphase abgeschloszen, jetzt darf auch der IO-Task loslaufen
			//		globalDOCKING.ready = TRUE;

			//		return (FALSE);

			//	}

			//} else {

			//	limit = 1;

			//}

			transmit[0] = (unsigned char)GetDeviceIDRemote();
			transmit[1] = 0x00;
			transmit[2] = 0x0C;
			transmit[3] = 0x21;
			transmit[4] = ((unsigned char *)&address)[3];
			transmit[5] = ((unsigned char *)&address)[2];
			transmit[6] = ((unsigned char *)&address)[1];
			transmit[7] = ((unsigned char *)&address)[0];
			transmit[8] = 0x00;
			transmit[9] = (count > bufSize) ? bufSize : count;

			repeat = limit;

			// je nach Timeboy oder Docking als Empfnger
			if (transmit[4] == 0x80) 
			{
				repeat = 2;
				pause = 80;
			} 
			else 
			{
				pause = 2000 / repeat;
			}

			for (i=0; i < repeat; i++) 
			{
				//if (i) DEBUGString(TRACE_DEVICE_DOCKING, "repeat <%u>.", i);

				if ((res = ExecuteDocking(transmit, 0x0A, receive, &rxCount)) == IV_ERROR_SUCCESS) 
				{
					// Fehlerhafte Ausfuehrungen zuruecksetzen
					executeFails = 0;

					if (transmit[4] == 0x80)
					{
						// wurde ein gltiges Antwortpaket empfangen, prfen, ob auch die Lnge paszt
						if (*(unsigned int *)&receive[4] == *(unsigned int *)&transmit[4]) 
						{
							break;
						}
					}
					else
					{
						if (rxCount == 7)
						{
							switch ( receive[4] )
							{
							case 0x22:
							case 0x23:
								break;
							case 0x21:
							case 0x24:
								{
									// Erster Befehl kann aufgrund von Sleep unbeantwortet bleiben.
									error21Cnt++;
									if (error21Cnt > 1)
									{
										return -IV_ERROR_UNREACHABLE;
									}
								}
								break;
							}
						}
						else
						{
							// Fehlerzaehler ruecksetzen.
							error21Cnt = 0;

							// wurde ein gueltiges Antwortpaket empfangen, pruefen, ob auch die Adresse passt
							if (*(unsigned int *)&receive[4] == *(unsigned int *)&transmit[4]) 
							{
								break;
							}
						}
					}
				}
				else
				{
					// Fehlerhafte Ausfuehrung
					executeFails++;
					if (executeFails >= receiveTrials)
					{
						return res;
					}
				}

				//KERNELMilliSleep(pause);
				Sleep(pause);
			}

			// bei erfolgreichem Senden keine Wiederholung
			if (i < repeat) 
			{
				break;
			}
		}

		// falls das Limit erreicht wurde, konnte der Befehl nicht erfolgreich gesendet werden!
		if (limit == 0) 
		{
			return -IV_ERROR_UNREACHABLE;
		}

		memcpy(buf, (unsigned char *)&receive[8], (count > bufSize) ? bufSize : count);

		buf += bufSize;
		address += bufSize;
		count -= ((count > bufSize) ? bufSize : count);
	}

	// Zeitpunkt der letzten Timeboykommunikation halten
	if ((address & 0x80000000) == 0)
	{
		m_timeboySlot[GetDeviceIDRemote()].SetReceiveStampTimeboy(clock());
	}

	return res;
}

