/**
 * @file  DFFileDeviceText.cpp
 * @brief Implementierung der Klasse CDFFileDeviceText.
 */
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DFFileDeviceText.h"
#ifndef _WIN32
#include <ctype.h>
#endif

CDFFileDeviceText::CDFFileDeviceText()
{
	// Member Initialisieren
	m_memData = NULL;
	m_memDefaultDFL = NULL;

	// Initialisieren
	Reset();
	// Fehlerobjekt zuruecksetzen
	ResetError();
}

CDFFileDeviceText::~CDFFileDeviceText()
{
	// Speicher freigeben
	if (m_memData) delete[] m_memData;
	m_memData = NULL;
	m_sizeData = 0;

	if (m_memDefaultDFL) delete[] m_memDefaultDFL;
	m_memDefaultDFL = NULL;
	m_sizeDefaultDFL = 0;
}

void CDFFileDeviceText::Reset()
{
	// Basis Reset
	CDFFileIntelHex::Reset();

	// Member Reseten
	if (m_memData) delete[] m_memData;
	m_memData = NULL;
	m_sizeData = 0;

	if (m_memDefaultDFL) delete[] m_memDefaultDFL;
	m_memDefaultDFL = NULL;
	m_sizeDefaultDFL = 0;
}

int CDFFileDeviceText::Load(char *fileName)
{
	// Lesen der Hexdatei
	FILE *fp;
	char data[0x400];

	// Zuruecksetzten
	Reset();
	// Fehlerobjekt zuruecksetzen
	ResetError();

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Daten eintragen
	unsigned char type;
	unsigned short address;
	unsigned char size;
	
	// Analyse
	// Speichergroesse ermitteln
	while(!feof(fp) && (m_errorState == ES_NO)) {

		if (fgets(data, 0x400, fp) != 0) {

			address = GetAddress(data);
			type    = GetType(data);
			size = GetCount(data);

			switch (type) {

			case INTELHEX_TYPE_TEXT_HEADER:
				break;

			case INTELHEX_TYPE_TEXT_DATA:
				if ((unsigned int)(address + size) > m_sizeData) {

					m_sizeData = address + size;
				}
				break;
			}
		}
	}

	// Auf Dateianfang setzen
	fseek(fp, 0, SEEK_SET);

	// Daten verfuegbar?
	if (m_sizeData) {

		// Speicher allocieren
		m_memData = new unsigned char[m_sizeData];
		memset(m_memData, 0, m_sizeData);

		// Daten einlesen
		while(!feof(fp) && (m_errorState == ES_NO)) {

			if (fgets(data, 0x400, fp) != 0) {

				address = GetAddress(data);
				type    = GetType(data);

				switch (type) {

				case INTELHEX_TYPE_TEXT_HEADER:
					break;

				case INTELHEX_TYPE_TEXT_DATA:
					GetData(data, &m_memData[address]);
					break;
				}
			}
		}
	}

	fclose(fp);	

	// Member durch gelesene Daten initialisieren und pruefen
	if (m_sizeData) {

		TEXTDATA *td = (TEXTDATA*)m_memData;
		// CRC32 pruefen
		if (td->crc != GetCRC32(&m_memData[td->ptrIDTable], td->cnt*sizeof(unsigned short) + td->size)) {

			// Fehler zuweisen
			m_errorState = ES_CRC_FAIL;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_CRC_FAIL_TEXT);
			return 0;
		}

	} else {

		// Fehler zuweisen
		m_errorState = ES_NO_DATA;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_NO_DATA_TEXT);
		return 0;
	}

	return 1;
}

int CDFFileDeviceText::Save(char *fileName, unsigned char append)
{
	FILE *fp;
	char szData[0x400];

	// Fehlerobjekt zuruecksetzen
	ResetError();

	if ((fp = fopen(fileName, (append) ? "a" : "w")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Daten der Zeichensaetze eintragen
	unsigned int ulPos;
	for (ulPos=0; ulPos < m_sizeData; ulPos += INTELHEX_DATA_COUNT) {

		if (!MakeTypedDataLine((unsigned short)ulPos, INTELHEX_TYPE_TEXT_DATA, 
			&m_memData[ulPos], (unsigned char)(((ulPos + INTELHEX_DATA_COUNT) < m_sizeData) ? INTELHEX_DATA_COUNT : m_sizeData - ulPos), szData)) {

				// Fehler zuweisen
				m_errorState = ES_INTEL_HEX;
				sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
				break;
		}

		fprintf(fp, "%s\n", szData);
	}

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		fclose(fp);
		return 0;
	}

	if (!append) {

		if (!MakeEndLine(szData)) {

			fclose(fp);
			// Fehler zuweisen
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			return 0;
		}

		fprintf(fp, "%s\n", szData);
	}

	fclose(fp);

	return 1;
}

char *CDFFileDeviceText::DefaultDFL()
{
	return m_memDefaultDFL;
}

// Import einer Textdatei mit folgendem Zeilenaufbau
// #define TEXT_SERIAL_NUMBER 10  ///< 1.2.3.1.1. Menuetext 1 fuer Seriennummer "serial number"
// ---------------------------**-------**************************************--*************-
int CDFFileDeviceText::Import(char *fileName)
{
	// Lesen der Header oder Default Datei
	FILE *fp;
	char data[0x400], dataTmp[0x400];
	unsigned short id;
	unsigned int flags, numLine = 0;
	char *format, *text;
	// Ist Headerfile?
	int fromHeader = (fileName[strlen(fileName) - 1] == 'h');

	// Fehlerobjekt zuruecksetzen
	ResetError();

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {

		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Speicher fuer Daten loeschen
	if (m_memData) delete[] m_memData;
	if (m_memDefaultDFL) delete[] m_memDefaultDFL;

	// Daten ermitteln
	unsigned short maxID = 0;
	size_t size = 0;
	unsigned int sizeDefaultDFL = 0;
	int res;

	// Analyse.
	// Maximalanzahl Texte, Gesamtgroesse Speicher
	while(!feof(fp) && (m_errorState == ES_NO)) {

		if (fgets(data, 0x400, fp) != 0) {

			// Zeilennummer
			numLine++;

			res = fromHeader ? ParseLineFromHeader(data, id, &format, &text, flags) : ParseLineFromDefault(data, id, &format, &text, flags);
			switch (res) {
				case 0: // Keine Auszuwertende Datenzeile
					break;

				case 1: // Ausgewertete Datenzeile
					// Dummies sind mit -1 (65535) angegeben
					if ((id != 65535) && id > maxID) maxID = id;

//sprintf(text, "%c%u", fromHeader ? 'D' : 'U', id);
					
					// Groesse fuer Default.dfl mitberechnen
					sizeDefaultDFL += sprintf(dataTmp, "%u\t%s\t\"%s\"\r\n", id, format, text);

					// Steuerzeichen binaer codieren
					ConvertControlCharacters(&text);

					// Nur Texte mit mindestens einem Zeichen werden auch uebertragen.
					if ((id != 65535) && strlen(text)) { 

						// Zu lange Texte abfangen, Maximal 130 Zeichen erlaubt.
						if (strlen(text) >= 130) {
							text[130] = 0;
						}

						size += strlen(text) + 1;
					}

#ifdef _WIN32
					TRACE("Zeile %u: %s\n", numLine, dataTmp);
#endif
					break;

				default: // Fehler bei Durchfuehrung
					fclose (fp);
					// Fehler zuweisen
					m_errorState = ES_LINE_FORMAT;
					sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_LINE_FORMAT_TEXT);
					return 0;
			}
		}
	}
	// Auf Dateianfang wechseln
	fseek(fp, 0, SEEK_SET);

	// Speicher fuer Textdaten gesamt allocieren
	m_sizeData = sizeof(TEXTDATA) + (maxID+1)*sizeof(unsigned short) + size;
	m_memData = new unsigned char[m_sizeData];
	if (m_memData == NULL) {
		
		// Fehler zuweisen
		m_errorState = ES_ALLOC_MEM;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_ALLOC_MEM_TEXT);
		return 0;
	}
	memset(m_memData, 0x00, m_sizeData);

	// Speicher fuer Default.dfl allocieren
	m_sizeDefaultDFL = sizeDefaultDFL;
	m_memDefaultDFL = new char[m_sizeDefaultDFL + 1];
	if (m_memDefaultDFL == NULL) {
		
		// Fehler zuweisen
		m_errorState = ES_ALLOC_MEM;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_ALLOC_MEM_TEXT);
		return 0;
	}
	memset(m_memDefaultDFL, 0x00, m_sizeDefaultDFL + 1);
	sizeDefaultDFL = 0;

	// Werte einlesen und aufbereiten
	TEXTDATA *tn = (TEXTDATA*)m_memData;
	tn->version = 1;
	strcpy(tn->name, "Textdaten");
	tn->type = 2;
	tn->cnt = maxID + 1;
	tn->size = (unsigned int)size;
	tn->ptrIDTable = sizeof(TEXTDATA);
	tn->ptrTextdata = tn->ptrIDTable + tn->cnt*sizeof(unsigned short);
	// Zeiger auf ersten freien Text
	unsigned short ptrText = (unsigned short)tn->ptrTextdata;

	while(!feof(fp) && (m_errorState == ES_NO)) {

		if (fgets(data, 0x400, fp) != 0) {

			res = fromHeader ? ParseLineFromHeader(data, id, &format, &text, flags) : ParseLineFromDefault(data, id, &format, &text, flags);
			switch (res) {
				case 0: // Keine auszuwertende Datenzeile
					break;

				case 1: // Ausgewertete Datenzeile
					// Dummies sind mit -1 (65535) angegeben
					if (id != 65535) {

//sprintf(text, "%c%u", fromHeader ? 'D' : 'U', id);

						// Daten gleich als Default.dfl aufbereiten
						sizeDefaultDFL += sprintf(&m_memDefaultDFL[sizeDefaultDFL], "%u\t%s\t\"%s\"\r\n", id, format, text);

						// Steuerzeichen binaer codieren
						ConvertControlCharacters(&text);

						if (strlen(text)) {

							// Zu lange Texte abfangen, Maximal 130 Zeichen erlaubt.
							if (strlen(text) >= 130) {
								text[130] = 0;
							}

							// Texte mit mindestens einem Zeichen Uebernehmen
							*(unsigned short*)&m_memData[tn->ptrIDTable + id*sizeof(unsigned short)] = ptrText;
							strcpy((char*)&m_memData[ptrText], text);
							ptrText += (unsigned short)strlen(text) + 1;

						} else {

							// Text als nicht vorhanden uebernehmen
							*(unsigned short*)&m_memData[tn->ptrIDTable + id*sizeof(unsigned short)] = 0;
						}

					}
					break;

				default: // Fehler bei Durchfuehrung
					fclose (fp);
					// Fehler zuweisen
					m_errorState = ES_LINE_FORMAT;
					sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_LINE_FORMAT_TEXT);
					return 0;
			}
		}
	}

	// CRC berechnen
	tn->crc = GetCRC32(&m_memData[tn->ptrIDTable], tn->cnt*sizeof(unsigned short) + tn->size);

	// Datei schliessen
	fclose(fp);

	return 1;
}

// Die Datenzeile hinter data wird manipuliert!!!
int CDFFileDeviceText::ParseLineFromHeader(char *data, unsigned short &id, char **format, char **text, unsigned int &flags)
{
	// Rueckgabeparameter initialisieren
	id = 65535;
	*format = 0;
	*text = 0;
	flags = 0;

	// Per Definition muss folgendes zutreffen
	// Zeile Beginnt mit #define, beinhaltet ///< und endet auf "\n\0
	if (data != strstr(data, "#define") || strstr(data, "///<") == NULL || data[strlen(data) - 2] != '\"' || data[strlen(data) - 1] != '\n') {
		// Keine gueltige Zeile fuer Textdaten
		return 0;
	}

	// suche nach Bloecken (id, format, text)
	char *pos, *textBegin;
	textBegin = NULL;

	// Aktuelle Position ist hinter #define 
	pos = &data[8];
	// weiterlaufen bis whitespace
	while(!isspace((unsigned char)*pos)) pos++;
	// weiterlaufen bis nach allen whitespaces
	while(isspace((unsigned char)*pos)) pos++;
	// ID extrahieren
	textBegin = pos; // Beginn der Zahl merken
	// weiterlaufen bis whitespace
	while(!isspace((unsigned char)*pos)) pos++;
	*pos = 0; // Terminierung hinter zahl
	id = (unsigned short)atoi(textBegin);
	// hinter dieser muss nun format folgen
	pos++;
	pos = strstr(pos, "///<");
	if (*pos == 0) {
		// Formatfehler
#ifdef _WIN32
		TRACE("ERROR: Zeile beinhaltet \"///<\" an falscher Position.\n");
#endif
		return -1;
	}
	pos += 4;
	// weiterlaufen bis nach allen whitespaces
	while(*pos && isspace((unsigned char)*pos)) pos++;
	*format = pos; // Beginn als Format halten
	if (!isdigit((unsigned char)*pos)) {
		// Formatfehler
#ifdef _WIN32
		TRACE("ERROR: Erstes Zeichen nach \"///<\" ist keine Ziffer.\n");
#endif
		return -1;
	}

	// Textbestand steht hinten
	pos = &pos[strlen(pos) - 2];
	*pos = 0; // Anfuehrungzeichen mit nullterminierung ueberschreiben
	pos--;
	// weiterlaufen bis " ohne \ davor
	while(*pos && (*pos != '"' || (*pos == '"' && *(pos-1) == '\\'))) pos--;
	if (*pos == 0) {
		// Formatfehler
#ifdef _WIN32
		TRACE("ERROR: Zeile endet nicht mit gueltigem String.\n");
#endif
		return -1;
	}
	*pos = 0; // Anfuehrungzeichen mit Nullterminierung ueberschreiben
	// Text extrahieren
	*text = pos + 1;

	// folgende whitespaces loeschen
	pos--;
	while(*pos && isspace((unsigned char)*pos)) { 
		*pos = 0; 
		pos--; 
	}

#ifdef _WIN32
	TRACE("ID: [%i], Format: [%s], Text: [%s]\n", id, *format, *text);
#endif

	return 1;
}

// Die Datenzeile hinter data wird manipuliert!!!
int CDFFileDeviceText::ParseLineFromDefault(char *data, unsigned short &id, char **format, char **text, unsigned int &flags)
{
	// Rueckgabeparameter initialisieren
	id = 65535;
	*format = 0;
	*text = 0;
	flags = 0;

	// Per Definition muss folgendes zutreffen
	// Zeile Beginnt mit Ziffer und endet auf "\n\0
	if (!isdigit((unsigned char)*data) || data[strlen(data) - 2] != '\"' || data[strlen(data) - 1] != '\n') {
		// Keine gueltige Zeile fuer Textdaten
		return 0;
	}

	// suche nach Bloecken (id, format, text)
	char *pos, *textBegin;
	textBegin = NULL;

	// Aktuelle Position
	pos = data;
	// ID extrahieren
	textBegin = pos; // Beginn der Zahl merken
	// weiterlaufen bis whitespace
	while(!isspace((unsigned char)*pos)) pos++;
	*pos = 0; // Terminierung hinter zahl
	id = (unsigned short)atoi(textBegin);
	// hinter dieser muss nun format folgen
	pos++;
	// weiterlaufen bis nach allen whitespaces
	while(*pos && isspace((unsigned char)*pos)) pos++;
	*format = pos; // Beginn als Format halten
	if (!isdigit((unsigned char)*pos)) {
		// Formatfehler
#ifdef _WIN32
		TRACE("ERROR: Erstes Zeichen des Formats ist keine Ziffer.\n");
#endif
		return -1;
	}

	// Textbestand steht hinten
	pos = &pos[strlen(pos) - 2];
	*pos = 0; // Anfuehrungzeichen mit nullterminierung ueberschreiben
	pos--;
	// weiterlaufen bis " ohne \ davor
	while(*pos && (*pos != '"' || (*pos == '"' && *(pos-1) == '\\'))) pos--;
	if (*pos == 0) {
		// Formatfehler
#ifdef _WIN32
		TRACE("ERROR: Zeile endet nicht mit gueltigem String.\n");
#endif
		return -1;
	}
	*pos = 0; // Anfuehrungzeichen mit Nullterminierung ueberschreiben
	// Text extrahieren
	*text = pos + 1;

	// folgende whitespaces loeschen
	pos--;
	while(*pos && isspace((unsigned char)*pos)) { 
		*pos = 0; 
		pos--; 
	}

#ifdef _WIN32
	TRACE("ID: [%i], Format: [%s], Text: [%s]\n", id, *format, *text);
#endif

	return 1;
}

void CDFFileDeviceText::ConvertControlCharacters(char **text)
{	
	// Kontrollsequenz vorhanden?
	if (strstr(*text, "\\r") != NULL || 
		strstr(*text, "\\t") != NULL) {

			// Replace von \r und \t
			char *src, *dest;
			dest = src = *text;

			while (*src) {

				if (*src == '\\') {

					if (*(src+1) == 'r') {

						*src = 0; src++; *src = 0;
						// Steuerzeichen \r
						*dest = '\r';
					} else if (*(src+1) == 't') {

						*src = 0; src++; *src = 0;
						// Steuerzeichen \t
						*dest = '\t';
					} else {

						// Kein Steuerzeichen
						*dest = *src;
						if (src > dest) *src = 0;
					}
				} else {

					*dest = *src;
					if (src > dest) *src = 0;
				}

				dest++;
				src++;
			}
	}
}
