// DFFileStudio.cpp: Implementierung der Klasse CDFFileStudio.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DFFileStudio.h"
#include <sys/stat.h>
#ifndef _WIN32
#include <string.h>
#include <time.h>
#endif
/*
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
*/
//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDFFileStudio::CDFFileStudio()
{
	// Speicher fuer EEP reservieren
	m_eep = new unsigned char[EEP_SIZE_SU];
  // Speicher fuer Dateiheader reservieren
  m_header = new unsigned char[EEP_SIZE - EEP_SIZE_SU];
	// Speicher fuer Flash reservieren
	m_flash = new unsigned char[FLASH_SIZE_SU + FLASH_SIZE_SU2 + EXTENDED_SIZE_SU + 100];
	if (m_flash == NULL) {
//		AfxMessageBox(_T("Nicht genuegend Speicher verfuegbar."), MB_ICONSTOP);
	}

  // Initialisieren der 32bit-CRC Umgebung.
  InitCRC32Table();

	// Dokument aufsetzen
	SetDefault();
}

CDFFileStudio::~CDFFileStudio()
{
	delete[] m_eep;
  delete[] m_header;
	delete[] m_flash;
}

void CDFFileStudio::SetDefault()
{
	// Eeprom 
	memset(m_eep, 0xFF, EEP_SIZE_SU);

  // Header auf Standard setzen.
  SetDefaultHeader();

	// Zusaetlicher Setupspeicher
	m_flashSizeSU2 = FLASH_SIZE_SU2;
	// Flashbereiche
	memset(m_flash, 0xFF, FLASH_SIZE_SU + FLASH_SIZE_SU2);

	// Konfigbereich
	memset(&m_config, 0, sizeof(TConfig));
	m_configMem = (unsigned char*)&m_config;

	// Extended Flash
	m_flashExtended = &m_flash[FLASH_SIZE_SU + FLASH_SIZE_SU2];
	memset(m_flashExtended, 0, EXTENDED_SIZE_SU + 100);

	// Der erweiterte Speicherbereich wird mit einer dynamischen Struktur begonnen,
	// deren erster Parameter ein Check - UINT32 - Wert enthaelt.
	TExtendedFlash extBegin;
	// Alles mit Null initialisieren.
	memset(&extBegin, 0, sizeof(TExtendedFlash));
	// Struktur fuer erweiterten Bereich aufsetzen
	extBegin.ulCheck = EXTENDED_CHECK;
	extBegin.ulSize  = sizeof(TExtendedFlash);
	extBegin.pulFreeScope = extBegin.ulSize;
	// Flag fuer Verarbeitung
	extBegin.ulFlags = EXTF_NEW;
	
	*(TExtendedFlash*)m_flashExtended = extBegin;

	// Listenpfade
	memset(m_listFileName, 0, sizeof(m_listFileName));
	memset(m_listEntranceFileName, 0, sizeof(m_listEntranceFileName));
	memset(m_listEntrance2FileName, 0, sizeof(m_listEntrance2FileName));
	memset(m_strExtendedFilePath, 0, sizeof(m_strExtendedFilePath));
}



void CDFFileStudio::SetDefaultHeader()
{
  // Dateiheader 
  memset(m_header, 0, EEP_SIZE - EEP_SIZE_SU);
  m_fileHeader = (TAESHeader*)m_header;
  // Dateikopf aufsetzen
  m_fileHeader->check = AESHEADER_CHECK;
  m_fileHeader->size = sizeof(TAESHeader);
  NeedStudioVersion("04.02.02.08");
  NeedLibraryVersion("04.02.02.08");

  // Es wird mindestens diese Version benoetigt.
  // Die Version wird ggf. auf die benoetigte in der jeweiligen MakeImage-Funktion angepasst.
  m_fileHeader->minFirmware1Version = 0;
  m_fileHeader->minFirmware2Version = 0;
}

int CDFFileStudio::Load(const char *fileName, int *error, int flags)
{
	int res = 1;
	struct stat st;
	char *fName;
	char mTimeStamp[20]; // 2009-05-19 09:16:22
	unsigned char buf[2]; // Laengenangabe bei Strings verarbeiten
	size_t length;
	int i;
	int listFileSize = 0;
	int entranceListFileSize = 0;
	int entrance2ListFileSize = 0;
	int extendedFilePathSize = 0;
	size_t maxHeaderSize = EEP_SIZE - EEP_SIZE_SU;
	unsigned int sizeSetup;
	struct tm *ptm;
  TExtendedFlash extBegin;

	if (stat(fileName, &st) != 0) {
    *error = -1;
		return 0;
	}

	FILE *fp = fopen(fileName, "rb");
	if (fp == NULL) {
#ifdef _WIN32
		TRACE("Setupdatei konnte nicht geoeffnet werden.\n");
#endif
    *error = -2;
		return 0;
	}

	// EEPROM - lesen
  if (fread(m_eep, sizeof(unsigned char), EEP_SIZE_SU, fp) != EEP_SIZE_SU) {
    *error = -3;
    res = 0;
    goto error_label;
  }
	
  // Dateiheader lesen.
  if (fread(m_header, sizeof(unsigned char), maxHeaderSize, fp) != maxHeaderSize) {
    *error = -3;
    res = 0;
    goto error_label;
  }
  if (IsHeader()) {

    // Header-CRC ueberpruefen.
    if (m_fileHeader->crc != GetCRC32(m_header, m_fileHeader->size - sizeof(unsigned int))) {
      *error = -4;
      res = 0;
      goto error_label;
    }

    // Mindestversion des Studio pruefen.
    if (!(flags & AESHEADER_IGNORE_MIN_STUDIO_VERSION) && m_fileHeader->minStudioVersion) {
      CDFSoftwarebuildVersion minVersion(m_fileHeader->minStudioVersion);
      if (minVersion > m_studioVersion) {
        *error = -5;
        res = 0;
        goto error_label;
      }
    }
    // Sollteversion des Studio pruefen?
    if (!(flags & AESHEADER_IGNORE_SHOULD_STUDIO_VERSION) && !(m_fileHeader->flags & AESHEADER_FLAG_IGNORE_SHOULD_STUDIO_VERSION) && m_fileHeader->shouldStudioVersion) {
      CDFSoftwarebuildVersion shouldVersion(m_fileHeader->shouldStudioVersion);
      if (shouldVersion > m_studioVersion) {
        *error = -6;
        res = 0;
        goto error_label;
      }
    }

    // Mindestversion der Library pruefen.
    if (!(flags & AESHEADER_IGNORE_MIN_LIBRARY_VERSION) && m_fileHeader->minLibraryVersion) {
      CDFSoftwarebuildVersion minVersion(m_fileHeader->minLibraryVersion);
      if (minVersion > m_libraryVersion) {
        *error = -7;
        res = 0;
        goto error_label;
      }
    }
    // Sollteversion der Library pruefen?
    if (!(flags & AESHEADER_IGNORE_SHOULD_LIBRARY_VERSION) && !(m_fileHeader->flags & AESHEADER_FLAG_IGNORE_SHOULD_LIBRARY_VERSION) && m_fileHeader->shouldLibraryVersion) {
      CDFSoftwarebuildVersion shouldVersion(m_fileHeader->shouldLibraryVersion);
      if (shouldVersion > m_libraryVersion) {
        *error = -8;
        res = 0;
        goto error_label;
      }
    }
  } else {

    // Den Header im Standard wiederherstellen.
    SetDefaultHeader();
  }

	// Setupgroesse
	sizeSetup = *(unsigned int*)&m_eep[P_FLASH_SIZE_SU];
	ChangeOrderLong((unsigned char*)&sizeSetup);
	// Ermittlung der Groesse des zu lesenden Speicherblocks fuer Flash.
	if ((*(unsigned short*)&m_eep[P_FILE_VERSION] == 0xDF02) && (sizeSetup > FLASH_SIZE_SU)) {

		m_flashSizeSU2 = FLASH_SIZE_SU2;
	} else {

		m_flashSizeSU2 = 0;
	}

	// Flashbereich - lesen
	if (fread(m_flash, sizeof(unsigned char), FLASH_SIZE_SU + m_flashSizeSU2, fp) != (FLASH_SIZE_SU + m_flashSizeSU2)) {
    *error = -3;
    res = 0;
    goto error_label;
  }

	// ListFile - lesen
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
		
		fread(buf, sizeof(unsigned char), 1, fp); // ein byte Laenge
		if (buf[0] == 0xFF) {

			fread(buf, sizeof(unsigned char), 2, fp); // zwei byte Laenge
			length = *(unsigned short*)buf;

			listFileSize += 3;
		} else {

			length = buf[0];
			listFileSize++;
		}
		listFileSize += length;

    if (fread(m_listFileName[i], sizeof(unsigned char), length, fp) != length) {
      *error = -3;
      res = 0;
      goto error_label;
    }
	}

	// Konfiguration - lesen
  if (fread(&m_config, sizeof(unsigned char), sizeof(TConfig), fp) != sizeof(TConfig)) {
    *error = -3;
    res = 0;
    goto error_label;
  }


	// Sind ab aktueller Postition noch 16kByte erweiterter Speicher vorhanden?
	if ((unsigned int)st.st_size >= 
		(EEP_SIZE_SU + maxHeaderSize + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(TConfig) + EXTENDED_SIZE_SU)) {

		// Erweiterter Bereich - lesen: 16kByte Extended Flash
    if (fread(m_flashExtended, sizeof(unsigned char), EXTENDED_SIZE_SU, fp) != EXTENDED_SIZE_SU) {
      *error = -3;
      res = 0;
      goto error_label;
    }
		// Pruefen der Extended - Struktur
		TExtendedFlash extBegin = *(TExtendedFlash*)m_flashExtended;

		if (extBegin.ulCheck != EXTENDED_CHECK) { // existiert?

      *error = -9;
      res = 0;
      goto error_label;
		}
  } else {

    *error = -9;
    res = 0;
    goto error_label;
  }
	
	// Sind noch ListFile fuer Zutrittskontrolle vorhanden?
	if ((unsigned int)st.st_size >= 
		(EEP_SIZE_SU + maxHeaderSize + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(TConfig) + EXTENDED_SIZE_SU + 4)) {
		
		// ListFile - lesen
		for (i=0; i < 4; i++) {
			
			fread(buf, sizeof(unsigned char), 1, fp); // ein byte laenge
			if (buf[0] == 0xFF) {

				fread(buf, sizeof(unsigned char), 2, fp); // zwei byte laenge
				length = *(unsigned short*)buf;

				entranceListFileSize += 3;
			} else {

				length = buf[0];
				entranceListFileSize++;
			}
			entranceListFileSize += length;

      if (fread(m_listEntranceFileName[i], sizeof(unsigned char), length, fp) != length) {
        *error = -3;
        res = 0;
        goto error_label;
      }
		}

		// Loeschen, wenn ZK nicht aktiviert.
		if (m_config.ENABLE_ENTRANCE == 0) {

			memset(m_listEntranceFileName, 0, sizeof(m_listEntranceFileName));
      // CRC von Pruefung ausschliessen.
      m_fileHeader->crcListEntranceFile = 0;
      // CRC ungueltig.
      m_fileHeader->crc = 0;
		}
  } else {

    *error = -9;
    res = 0;
    goto error_label;
  }

	// Sind noch ListFile fuer Zutrittskontrolle 2 vorhanden?
	if ((unsigned int)st.st_size >= 
		(EEP_SIZE_SU + maxHeaderSize + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(TConfig) + EXTENDED_SIZE_SU + entranceListFileSize + MAX_DATENSATZ_BESCHREIBUNG)) {

		// ListFile - lesen
		for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
			
			fread(buf, sizeof(unsigned char), 1, fp); // ein byte laenge
			if (buf[0] == 0xFF) {

				fread(buf, sizeof(unsigned char), 2, fp); // zwei byte laenge
				length = *(unsigned short*)buf;

				entrance2ListFileSize += 3;
			} else {

				length = buf[0];
				entrance2ListFileSize++;
			}
			entrance2ListFileSize += length;

      if (fread(m_listEntrance2FileName[i], sizeof(unsigned char), length, fp) != length) {
        *error = -3;
        res = 0;
        goto error_label;
      }
    }

		// Loeschen, wenn ZK nicht aktiviert.
		if (m_config.ENABLE_ENTRANCE2 == 0) {

			memset(m_listEntrance2FileName, 0, sizeof(m_listEntrance2FileName));
      // CRC von Pruefung ausschliessen.
      m_fileHeader->crcListEntrance2File = 0;
      // CRC ungueltig.
      m_fileHeader->crc = 0;
		}
  } else {
/*
    *error = -9;
    res = 0;
    goto error_label;
*/
    // (BO)Bugfix: 2012_03_19
    // Diese Dateiangaben muessen noch nicht verfuegbar sein.
    // Es koennten Dateien fuer AEIII+ geladen werden!
  }

	// Sind noch ExtendedFilePath fuer Zusatzdateien vorhanden?
	if ((unsigned int)st.st_size >= 
		(EEP_SIZE_SU + maxHeaderSize + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(TConfig) + EXTENDED_SIZE_SU + entranceListFileSize + entrance2ListFileSize + MAX_DATENSATZ_BESCHREIBUNG)) {

		// ListFile - lesen
		for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
			
			fread(buf, sizeof(unsigned char), 1, fp); // ein byte laenge
			if (buf[0] == 0xFF) {

				fread(buf, sizeof(unsigned char), 2, fp); // zwei byte laenge
				length = *(unsigned short*)buf;

				extendedFilePathSize += 3;
			} else {

				length = buf[0];
				extendedFilePathSize++;
			}
			extendedFilePathSize += length;

      if (fread(m_strExtendedFilePath[i], sizeof(unsigned char), length, fp) != length) {
        *error = -3;
        res = 0;
        goto error_label;
      }
		}
  } else {

    // Diese Dateiangaben muessen noch nicht verfuegbar sein.
    // Es koennen Dateien ab DatafoxStudioIV - Version 04.01.01.10 geladen werden!
  }

  if (IsHeader()) {

    // CRC der Dateibereiche pruefen
    if (m_fileHeader->crcEep && m_fileHeader->crcEep != GetCrcEep()) {
      *error = -10;
      res = 0;
      goto error_label;
    }
    m_fileHeader->crcEep = 0;
    
    if (m_fileHeader->crcEepExtended && m_fileHeader->crcEepExtended != GetCrcEepExtended()) {
      *error = -11;
      res = 0;
      goto error_label;
    }
    m_fileHeader->crcEepExtended = 0;

    if (m_fileHeader->crcFlash && m_fileHeader->crcFlash != GetCrcFlash()) {
      *error = -12;
      res = 0;
      goto error_label;
    }
    m_fileHeader->crcFlash = 0;

    if (m_fileHeader->crcListFile && m_fileHeader->crcListFile != GetCrcListFile()) {
      *error = -13;
      res = 0;
      goto error_label;
    }
    m_fileHeader->crcListFile = 0;

    if (m_fileHeader->crcConfig && m_fileHeader->crcConfig != GetCrcConfig()) {
      *error = -14;
      res = 0;
      goto error_label;
    }
    m_fileHeader->crcConfig = 0;

    if (m_fileHeader->crcFlashExtended && m_fileHeader->crcFlashExtended != GetCrcFlashExtended()) {
      *error = -15;
      res = 0;
      goto error_label;
    }
    m_fileHeader->crcFlashExtended = 0;

    if (m_fileHeader->crcListEntranceFile && m_fileHeader->crcListEntranceFile != GetCrcListEntranceFile()) {
      *error = -16;
      res = 0;
      goto error_label;
    }
    m_fileHeader->crcListEntranceFile = 0;

    if (m_fileHeader->crcListEntrance2File && m_fileHeader->crcListEntrance2File != GetCrcListEntrance2File()) {
      *error = -17;
      res = 0;
      goto error_label;
    }
    m_fileHeader->crcListEntrance2File = 0;

    if (m_fileHeader->crcListExtendedFile && m_fileHeader->crcListExtendedFile != GetCrcListExtendedFile()) {
      *error = -18;
      res = 0;
      goto error_label;
    }
    m_fileHeader->crcListExtendedFile = 0;
  }
	// Die Werte fuer Dateinamen und Zeitstempel der Modifizierung eintragen.
	// Dateinamen extrahieren
	fName = (char*)&fileName[strlen(fileName)];
	while (fName > fileName) {

		// Maskierte Leerzeichen (unix)
		if (*(fName - 1) == '\\' && *fName == ' ') {

		} else if(*(fName - 1) == '\\') {

			// Pfadbestandteil vor Dateinamen (windows)
			break;

		} else if (*(fName - 1) == '/') {

			// Pfadbestandteil vor Dateinamen (unix)
			break;
		} 

		fName--;
	}

	ptm = localtime(&st.st_mtime);
	sprintf(mTimeStamp, "%02i%02i-%02i-%02i %02i:%02i:%02i", 
		(ptm->tm_year + 1900) / 100, (ptm->tm_year + 1900) % 100, ptm->tm_mon + 1, ptm->tm_mday,
		ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
	
	// Eintragen in der Extended - Struktur
	extBegin = *(TExtendedFlash*)m_flashExtended;
	if (extBegin.ulCheck == EXTENDED_CHECK && extBegin.pulExtended) {

		TExtended *extended = (TExtended*)&m_flashExtended[extBegin.pulExtended];
		strncpy(extended->fileName, fName, sizeof(extended->fileName));
		// Nullterminierung sicherstellen
		extended->fileName[sizeof(extended->fileName) - 1] = 0;
		strcpy(extended->fileTimeStamp, mTimeStamp);
	}

error_label:
  if (fp) fclose(fp);

	return res;
}

int CDFFileStudio::Save(const char *fileName, int *error)
{
	int res = 1;
	int i;
	unsigned int sizeSetup;

  *error = 0;
	FILE *fp = fopen(fileName, "wb");
	if (fp == NULL) {
#ifdef _WIN32
		TRACE("Setupdatei konnte nicht geoeffnet werden.\n");
#endif
		return 0;
	}

	// Groesse der Setupdaten eintragen
	sizeSetup = GetFlashSetupSize();
	*(unsigned int*)&m_eep[P_FLASH_SIZE_SU] = sizeSetup;
	ChangeOrderLong(&m_eep[P_FLASH_SIZE_SU]);

	// Ermittlung der Groesse des zu schreibenden Speicherblocks fuer Flash.
	if (sizeSetup > FLASH_SIZE_SU) {

		m_flashSizeSU2 = FLASH_SIZE_SU2;
		*(unsigned short*)&m_eep[P_FILE_VERSION] = 0xDF02;
	} else {

		m_flashSizeSU2 = 0;
		*(unsigned short*)&m_eep[P_FILE_VERSION] = 0xDF01;
	}

	// EEPROM - schreiben
	if (fwrite(m_eep, sizeof(unsigned char), EEP_SIZE_SU, fp) != EEP_SIZE_SU) {
#ifdef _WIN32
		TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
		res = 0;
		goto error_label;
	}

  // CRC der Dateibereiche
  m_fileHeader->crcEep = GetCrcEep();
  m_fileHeader->crcEepExtended = GetCrcEepExtended(); // CRC von Restheaderbereich berechnen und eintragen
  m_fileHeader->crcFlash = GetCrcFlash();
  m_fileHeader->crcListFile = GetCrcListFile();
  m_fileHeader->crcConfig = GetCrcConfig();
  m_fileHeader->crcFlashExtended = GetCrcFlashExtended();
  m_fileHeader->crcListEntranceFile = GetCrcListEntranceFile();
  m_fileHeader->crcListEntrance2File = GetCrcListEntrance2File();
  m_fileHeader->crcListExtendedFile = GetCrcListExtendedFile();
  // Gesamt-CRC fuer Header berechnen und eintragen.
  m_fileHeader->crc = GetCRC32((unsigned char*)m_fileHeader, m_fileHeader->size - sizeof(unsigned int));

  // Dateiheaderbereich schreiben
  if (fwrite(m_header, sizeof(unsigned char), EEP_SIZE - EEP_SIZE_SU, fp) != (EEP_SIZE - EEP_SIZE_SU)) {
#ifdef _WIN32
    TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
    res = 0;
    goto error_label;
  }

	// Flashbereich - schreiben
	if (fwrite(m_flash, sizeof(unsigned char), FLASH_SIZE_SU + m_flashSizeSU2, fp) != (FLASH_SIZE_SU + m_flashSizeSU2)) {
#ifdef _WIN32
		TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
		res = 0;
		goto error_label;
	}

	// ListFile - schreiben
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

		// Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
		if (strlen(m_listFileName[i]) > 0xFF) {

			unsigned char len[3];
			len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
			*(unsigned short*)&len[1] = (unsigned short)strlen(m_listFileName[i]);
			if (fwrite(len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len)) {
#ifdef _WIN32
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
				res = 0;
				goto error_label;
			}
		} else {

			unsigned char len = (unsigned char)strlen(m_listFileName[i]);
			if (fwrite(&len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len)) {
#ifdef _WIN32
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
				res = 0;
				goto error_label;
			}
		}

		// String rausschreiben
		if (fwrite(m_listFileName[i], sizeof(unsigned char), strlen(m_listFileName[i]), fp) != strlen(m_listFileName[i])) {
#ifdef _WIN32
			TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
			res = 0;
			goto error_label;
		}
	}

	// Konfiguration - schreiben
	if (fwrite(&m_config, sizeof(unsigned char), sizeof(TConfig), fp) != sizeof(TConfig)) {
#ifdef _WIN32
		TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
		res = 0;
		goto error_label;
	}

	// Erweiterter Speicher - schreiben
	if (fwrite(m_flashExtended, sizeof(unsigned char), EXTENDED_SIZE_SU, fp) != EXTENDED_SIZE_SU) {
#ifdef _WIN32
		TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
		res = 0;
		goto error_label;
	}

	// Zutrittskontrolle - ListFile - schreiben
	for (i=0; i < 4; i++) {

		// Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
		if (strlen(m_listEntranceFileName[i]) > 0xFF) {

			unsigned char len[3];
			len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
			*(unsigned short*)&len[1] = (unsigned short)strlen(m_listEntranceFileName[i]);
			if (fwrite(len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len)) {
#ifdef _WIN32
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
				res = 0;
				goto error_label;
			}
		} else {

			unsigned char len = (unsigned char)strlen(m_listEntranceFileName[i]);
			if (fwrite(&len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len)) {
#ifdef _WIN32
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
				res = 0;
				goto error_label;
			}
		}

		// String rausschreiben
		if (fwrite(m_listEntranceFileName[i], sizeof(unsigned char), strlen(m_listEntranceFileName[i]), fp) != strlen(m_listEntranceFileName[i])) {
#ifdef _WIN32
			TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
			res = 0;
			goto error_label;
		}
	}
	// Zutrittskontrolle - ListFile - schreiben
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

		// Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
		if (strlen(m_listEntrance2FileName[i]) > 0xFF) {

			unsigned char len[3];
			len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
			*(unsigned short*)&len[1] = (unsigned short)strlen(m_listEntrance2FileName[i]);
			if (fwrite(len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len)) {
#ifdef _WIN32
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
				res = 0;
				goto error_label;
			}
		} else {

			unsigned char len = (unsigned char)strlen(m_listEntrance2FileName[i]);
			if (fwrite(&len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len)) {
#ifdef _WIN32
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
				res = 0;
				goto error_label;
			}
		}

		// String rausschreiben
		if (fwrite(m_listEntrance2FileName[i], sizeof(unsigned char), strlen(m_listEntrance2FileName[i]), fp) != strlen(m_listEntrance2FileName[i])) {
#ifdef _WIN32
			TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
			res = 0;
			goto error_label;
		}
	}

	// Zusatzpfadangaben - schreiben
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

		// Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
		if (strlen(m_strExtendedFilePath[i]) > 0xFF) {

			unsigned char len[3];
			len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
			*(unsigned short*)&len[1] = (unsigned short)strlen(m_strExtendedFilePath[i]);
			if (fwrite(len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len)) {
#ifdef _WIN32
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
				res = 0;
				goto error_label;
			}
		} else {

			unsigned char len = (unsigned char)strlen(m_strExtendedFilePath[i]);
			if (fwrite(&len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len)) {
#ifdef _WIN32
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
				res = 0;
				goto error_label;
			}
		}

		// String rausschreiben
		if (fwrite(m_strExtendedFilePath[i], sizeof(unsigned char), strlen(m_strExtendedFilePath[i]), fp) != strlen(m_strExtendedFilePath[i])) {
#ifdef _WIN32
			TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
#endif
			res = 0;
			goto error_label;
		}
	}

error_label:
	if (fp) fclose(fp);

	return res;
}

//########################
// CRC der Dateibereiche
//########################

///< CRC ueber Eep-Bereich.
unsigned int CDFFileStudio::GetCrcEep() {
  return GetCRC32(m_eep, EEP_SIZE_SU);
}

///< CRC ueber EepExtended-Bereich.
unsigned int CDFFileStudio::GetCrcEepExtended() {
  return GetCRC32(&m_header[m_fileHeader->size], EEP_SIZE - EEP_SIZE_SU - m_fileHeader->size);
}

///< CRC ueber Flash-Bereich.
unsigned int CDFFileStudio::GetCrcFlash() {
  // Groesse der Setupdaten
  unsigned int sizeSetup = GetFlashSetupSize();

  // Ermittlung der Groesse des zu beachtenden Speicherblocks fuer Flash.
  if (sizeSetup > FLASH_SIZE_SU) {

    m_flashSizeSU2 = FLASH_SIZE_SU2;
  }

  return GetCRC32(m_flash, FLASH_SIZE_SU + m_flashSizeSU2);
}

///< CRC ueber die Listendateiangaben.
unsigned int CDFFileStudio::GetCrcListFile() {
  unsigned int crc = 0;
  int i;
  for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

    // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
    if (strlen(m_listFileName[i]) > 0xFF) {

      unsigned char len[3];
      len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
      *(unsigned short*)&len[1] = (unsigned short)strlen(m_listFileName[i]);
      crc = GetCRC32(len, sizeof(len), crc, 0);
    } else {

      unsigned char len = (unsigned char)strlen(m_listFileName[i]);
      crc = GetCRC32(&len, sizeof(len), crc, 0);
    }

    // String rausschreiben
    crc = GetCRC32((unsigned char*)m_listFileName[i], strlen(m_listFileName[i]), crc, (i == (MAX_DATENSATZ_BESCHREIBUNG - 1)));
  }

  return crc;
}

///< CRC ueber die Config-Struktur.
unsigned int CDFFileStudio::GetCrcConfig() {
  return GetCRC32((unsigned char*)&m_config, sizeof(TConfig));
}

///< CRC ueber FlashExtended-Bereich.
unsigned int CDFFileStudio::GetCrcFlashExtended() {
  return GetCRC32(m_flashExtended, EXTENDED_SIZE_SU);
}

 ///< CRC ueber die Listendateiangaben der ZK1.
unsigned int CDFFileStudio::GetCrcListEntranceFile() {
  unsigned int crc = 0;
  int i;
  for (i=0; i < 4; i++) {

    // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
    if (strlen(m_listEntranceFileName[i]) > 0xFF) {

      unsigned char len[3];
      len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
      *(unsigned short*)&len[1] = (unsigned short)strlen(m_listEntranceFileName[i]);
      crc = GetCRC32(len, sizeof(len), crc, 0);
    } else {

      unsigned char len = (unsigned char)strlen(m_listEntranceFileName[i]);
      crc = GetCRC32(&len, sizeof(len), crc, 0);
    }

    // String rausschreiben
    crc = GetCRC32((unsigned char*)m_listEntranceFileName[i], strlen(m_listEntranceFileName[i]), crc, (i == (4 - 1)));
  }

  return crc;
}

///< CRC ueber die Listendateiangaben der ZK2.
unsigned int CDFFileStudio::GetCrcListEntrance2File() {
  unsigned int crc = 0;
  int i;
  for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

    // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
    if (strlen(m_listEntrance2FileName[i]) > 0xFF) {

      unsigned char len[3];
      len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
      *(unsigned short*)&len[1] = (unsigned short)strlen(m_listEntrance2FileName[i]);
      crc = GetCRC32(len, sizeof(len), crc, 0);
    } else {

      unsigned char len = (unsigned char)strlen(m_listEntrance2FileName[i]);
      crc = GetCRC32(&len, sizeof(len), crc, 0);
    }

    // String rausschreiben
    crc = GetCRC32((unsigned char*)m_listEntrance2FileName[i], strlen(m_listEntrance2FileName[i]), crc, (i == (MAX_DATENSATZ_BESCHREIBUNG - 1)));
  }

  return crc;
}

///< CRC ueber die Zusaetzlichen Listendateiangaben.
unsigned int CDFFileStudio::GetCrcListExtendedFile() {
  unsigned int crc = 0;
  int i;
  for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

    // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
    if (strlen(m_strExtendedFilePath[i]) > 0xFF) {

      unsigned char len[3];
      len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
      *(unsigned short*)&len[1] = (unsigned short)strlen(m_strExtendedFilePath[i]);
      crc = GetCRC32(len, sizeof(len), crc, 0);
    } else {

      unsigned char len = (unsigned char)strlen(m_strExtendedFilePath[i]);
      crc = GetCRC32(&len, sizeof(len), crc, 0);
    }

    // String rausschreiben
    crc = GetCRC32((unsigned char*)m_strExtendedFilePath[i], strlen(m_strExtendedFilePath[i]), crc, (i == (MAX_DATENSATZ_BESCHREIBUNG - 1)));
  }

  return crc;
}

// Die zu verwendende Version der Datei
int CDFFileStudio::NeededVersion()
{
	if (GetFlashSetupSize() > FLASH_SIZE_SU) {

		return 2;
	}

	return 1;
}

void CDFFileStudio::ChangeOrderLong(unsigned char *Buf)
{
	unsigned char h;

	h = Buf[0];
	Buf[0] = Buf[3];
	Buf[3] = h;

	h = Buf[1];
	Buf[1] = Buf[2];
	Buf[2] = h;
}

// Groesse der Setupdaten im Flash ermitteln
unsigned int CDFFileStudio::GetFlashSetupSize()
{
	// ##############################################################
	// Pointer sind direkte Adressen die im AEIII+:
	//   1. Im Motorolla - Format vorliegen.
	//   2. Mit einem Basisoffset -FLASH_KONFIG_START- berechnet sind.

	// Pointer auf ListenBeschreibungen ermitteln
	unsigned int pLB = *(unsigned int*)(&m_eep[P_LIST_STRUCT]);

	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pLB);
	//}
	pLB -= FLASH_KONFIG_START;

	// Pointerarray der Listenbeschreibungen durchschreiten
	int i;
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
	
		// Nullpointer suchen fuer Listenanzahl
		if (*(unsigned int*)&m_flash[pLB + i * sizeof(unsigned int)] == 0ul) {
		
			break;
		}
	}

	// Liste gefunden?

	if (i > 0) {
	
		// Pointer auf letzte definierte Liste
		pLB = *(unsigned int*)(&m_flash[pLB + (i - 1) * sizeof(unsigned int)]);

		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pLB);
		//}
		pLB -= FLASH_KONFIG_START;

		TDatensatzBeschreibung *pLBStruct = (TDatensatzBeschreibung*)(&m_flash[pLB]);

		// Pointer auf Feldarray der Liste
		unsigned int pFeldArray = pLBStruct->P;
		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pFeldArray);
		//}
		pFeldArray -= FLASH_KONFIG_START;

		// Felder vorhanden?
		if (pLBStruct->Felder) {
		
			// Pointer auf letztes Feld der Listenbeschreibung ermitteln.
			unsigned int pFeld = *(unsigned int*)&m_flash[pFeldArray + (pLBStruct->Felder - 1) * sizeof(unsigned int)];
			//if (App->ByteOrder == BO_MOT) {
				
				ChangeOrderLong((unsigned char*)&pFeld);
			//}
			pFeld -= FLASH_KONFIG_START;
			
			// Feldadresse + eigene Groesse bildet FlashSetupSize
			return pFeld + sizeof(TDatensatzFeld);
		}


		// Keine Felder definiert!
		return pFeldArray;
	}
	
	// Keine Listenbeschreibung definiert!
	// Die Anzahl fuer Listenbeschreibungspointer muss mitberechnet werden,
	// um bei einer Uebertragung diese zu loeschen.
	return pLB + MAX_DATENSATZ_BESCHREIBUNG * sizeof(unsigned int);
}

int CDFFileStudio::ValidFirmwareVersion(CDFSoftwarebuildVersion &fwVersion)
{
  if (IsHeader()) {
    // Pruefung auf 04.01.xx
    if (fwVersion >= "04.01.01.00" && fwVersion < "04.02.00.00") {

      // Unsupported der Firmware 04.01.xx.yy pruefen.
      if (m_fileHeader->minFirmware1Version == AESHEADER_VERSION_UNSUPPORTED) {
        return -1;
      }
      // Mindestversion der Firmware 04.01.xx.yy pruefen.
      if (m_fileHeader->minFirmware1Version) {
        CDFSoftwarebuildVersion minVersion(m_fileHeader->minFirmware1Version);
        if (minVersion > fwVersion) {
          return -2;
        }
      }
      // Sollteversion des der Firmware 04.01.xx.yy pruefen?
      if (!(m_fileHeader->flags & AESHEADER_FLAG_IGNORE_SHOULD_FIRMWARE1_VERSION) && m_fileHeader->shouldFirmware1Version) {
        CDFSoftwarebuildVersion shouldVersion(m_fileHeader->shouldFirmware1Version);
        if (shouldVersion > fwVersion) {
          return -3;
        }
      }
    } else if (fwVersion >= "04.02.00.00" && fwVersion < "04.03.00.00") {

      // Unsupported der Firmware 04.02.xx.yy pruefen.
      if (m_fileHeader->minFirmware2Version == AESHEADER_VERSION_UNSUPPORTED) {
        return -4;
      }
      // Mindestversion der der Firmware 04.02.xx.yy pruefen.
      if (m_fileHeader->minFirmware2Version) {
        CDFSoftwarebuildVersion minVersion(m_fileHeader->minFirmware2Version);
        if (minVersion > fwVersion) {
          return -5;
        }
      }
      // Sollteversion der der Firmware 04.02.xx.yy pruefen?
      if (!(m_fileHeader->flags & AESHEADER_FLAG_IGNORE_SHOULD_FIRMWARE2_VERSION) && m_fileHeader->shouldFirmware2Version) {
        CDFSoftwarebuildVersion shouldVersion(m_fileHeader->shouldFirmware2Version);
        if (shouldVersion > fwVersion) {
          return -6;
        }
      }
    } else {

      // Die Firmwareversion wird nicht untersttzt.
      return -7;
    }

    // Firmwareversion wird unterstuetzt.
    return 1;
  }

  // Es liegt kein Dateiheader vor!
  return 0;
}
