#ifndef _CHANNEL4_MODULE_APPLICATION_H_
#define _CHANNEL4_MODULE_APPLICATION_H_

// Basisklasse einbinden
#include "Channel4ModuleSystem.h"

// MODULE_APPLICATION( 2 )

#define IV_MODULE_APPLICATION                    0x0002

// Fehler:
// 1 -> Modul unbekannt
// 2 -> Funktion unbekannt
// 3 -> Zu wenige Parameter
// 4 -> Zu viele Parameter

// Clear: Alle Datensaetze loeschen. CLEAR_RECORDS( 0 )
#define IV_FUNCTION_CLEAR_RECORDS                0x0000
// Set: Datensatz quittieren. SET_RECORD( 1 )
#define IV_FUNCTION_SET_RECORD                   0x0001
// Get: Datensatz hohlen. GET_RECORD( 2 )
#define IV_FUNCTION_GET_RECORD                   0x0002

// Datensatzanfrage
//#define MB_CMD_READ_RECORD              0x40  /* Datensatz senden */

#define MB_ANSW_NO_RECORD               0xC0  /* kein Datensatz vorhanden */
#define MB_ANSW_OFFLINE_RECORD          0xC1  /* Offline Datensatz */
#define MB_ANSW_OLD_OFFLINE_RECORD      0xC3  /* wiederholt gesendeter Offline Datensatz */
#define MB_ANSW_ONLINE_RECORD           0xE1  /* Online Datensatz */
#define MB_ANSW_OLD_ONLINE_RECORD       0xE3  /* wiederholt gesendeter Online Datensatz */

// Datensatzquittierung
//#define MB_CMD_ACK_RECORD               0x48  /* Quittung fuer Datensatz */

#define MB_ANSW_ACK_OFF_RECORD	        0xC8  /* Quittung erhalten (fuer Offline-Datensatz) */
#define MB_ANSW_OLD_ACK_OFF_RECORD      0xCA  /* wiederholt gesendete Quittung */
#define MB_ANSW_ACK_ON_RECORD           0xE8  /* Quittung erhalten (fuer Online-Datensatz) */
#define MB_ANSW_OLD_ACK_ON_RECORD       0xEA  /* wiederholt gesendete Quittung */
#define MB_ANSW_TIMEOUT_ACK_RECORD      0xE9  /* Quittung kam nach Timeout (fuer Online-Datensatz) */
#define MB_ANSW_OLD_TIMEOUT_ACK_RECORD  0xEB  /* wiederholte Quittung nach Timeout */

// Datensatzloeschbefehl
//#define MB_CMD_DEL						0x49  /* Loescht alle Datensaetze. */



// Set: Message SHOW_MESSAGE( 3 )
#define IV_FUNCTION_SHOW_MESSAGE                 0x0003
// (UIN8(Time)unsigned char(Action)unsigned char(Audio)Message\0)
// -> ()
// Time ist die anzuzeigende Dauer
// Action ist die durchzufuehrende Aktion (unbenutzt und immer 0)
// Audio ist das zu verwendende akustische Signal
// Message ist der eigentliche Nachrichtentext

// Fehler:
// 1 -> Nachrichtentext zu lang fuer die Anzeige.
// 2 -> Es wird bereits eine Nachricht angezeigt.


// Set: Variable (Setup) SET_VARIABLE( 4 )
#define IV_FUNCTION_SET_VARIABLE                 0x0004
// Get: Variable (Setup) GET_VARIABLE( 5 )
#define IV_FUNCTION_GET_VARIABLE                 0x0005

// Der Name einer Variablen setzt sich auf folgende Art zusammen:
// MAJOR_NAME.MINOR_NAME
// Der Majorname einer Setupvariablen ist immer SETUP.
// Fehlernummer:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = Lnge des Variablenwertes ist zu lang.

#define IV_FUNCTION_SHOW_INFO                    0x0006
// Maschinennummer (1 Byte)
// Time ist die Anzeigedauer. (1 Byte) (0 im Hindergrund uebernehmen, -1 staendig anzeigen)
// Schriftgroessennummer (1 Byte)
// Akkustisches Signal (1 Byte)
// 16bit reserve (2 Byte)
// Flags z.b. 0x0001 (2 Byte) fuer Antiproportionale Schrift
// Infotext

#define IV_FUNCTION_ENTRANCE2_IDENTIFICATION     0x0007
// Alle Parameter mit fester Laenge. Linksausrichtung.
// param1: 4 Zeichen MasterID
// param2: 3 Zeichen Tuermodul
// param3: 20 Zeichen Ausweis
// param4: 8 Zeichen Pin

// Aktive Antwort: Status der Zutrittspruefung als Parameter
#define IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK  0x0008

// Benachrichtigung ueber ein Ereignis
#define IV_FUNCTION_NOTIFY                       0x0009

// Mindestens ein Datensatz liegt vor.
#define IV_VALUE_NOTIFY_RECORD                   0x00


class CRecord
{
public:
	CRecord() 
	{
		m_priority = 0;
		m_status = 0;
		memset(&m_timestamp, 0, sizeof(tm));
		m_delay = 0;
		m_length = 0;
		memset(m_data, 0, sizeof(m_data));
	}
	int m_priority;
	int m_status;
	struct tm m_timestamp;
	int m_delay;
	int m_length;
	unsigned char m_data[256];
};

// Kontainerklassen fuer Prioritaetswarteschlange importieren.
#include <deque>
#include <queue>

typedef std::deque<CRecord> record_deque;
typedef std::queue<CRecord> record_queue;

class CChannel4ModuleApplication : public CChannel4ModuleSystem
{
public:
	/// Konstruktor fuer Initialisierung der Klasse.
	CChannel4ModuleApplication()  : CChannel4ModuleSystem(IV_DEFAULT_CHANNELID, IV_DEFAULT_DEVICEID)
	{ InitClass(); }
	CChannel4ModuleApplication(short channelID, short deviceID) : CChannel4ModuleSystem(channelID, deviceID) 
	{ InitClass(channelID, deviceID); }
	virtual void InitClass(short channelID = IV_DEFAULT_CHANNELID, short deviceID = IV_DEFAULT_DEVICEID);
	/// Destruktor
	virtual ~CChannel4ModuleApplication();

public:
	// Gibt Modulfehler zurueck.
	virtual void GetErrorModule(short module, short error, char *text);
	// Gibt Funktionsfehlertext zurueck.
	virtual void GetErrorFunction(short module, short function, short error, char *text);

	// MODULE_APPLICATION

	// Datensaetze loeschen
	int ClearRecords(short deviceIDRemote);

	// Datensatz quittieren/anfordern
	int SetRecord(short deviceIDRemote, unsigned char &status);
	int GetRecord(short deviceIDRemote, unsigned char *buf, int &count, unsigned char &status);

	// Displaynachicht senden
	int ShowMessage(short deviceIDRemote, unsigned char time, unsigned char action, unsigned char audio, char *text);

	// Variable setzen/lesen
	int SetVariable(short deviceIDRemote, char *name, char *value);
	int GetVariable(short deviceIDRemote, char *name, char *value);

	// Info senden
	int ShowInfo(short deviceIDRemote, unsigned char maschine, unsigned char time, unsigned char font, unsigned char audio, short flags, char *text);

	// Zutrittsbuchung durchfuehren
	int Entrance2Identification(short deviceIDRemote, char *TM, char *Id, char *Pin, char *status, int &statusLength);

	// Eine Benachrichtigung melden
	int Notify(short deviceIDRemote, unsigned char type, int priority, unsigned char deviceType, unsigned long serialNum, char *commString, int commValue, char *params);
	int NotifyRecord(short deviceIDRemote, int priority, unsigned char deviceType, unsigned long serialNum, char *commString, int commValue);

	//#################################
	// CallRoutinen

	// Ausfuehren von Befehlen
	virtual int CallFunction(short module, short function, unsigned char *data, int &count);

	// Zutrittsbuchung durchfuehren (aktive Antwort)
	int CallEntrance2IdentificationOK(unsigned char *data, int &count);

	// Eine Benachrichtigung melden
	int CallNotify(unsigned char *data, int &count);

	// Datensaetze loeschen
	int CallClearRecords(unsigned char *data, int &count);

	record_queue m_records;
	// Datensatz quittieren/anfordern
	int CallSetRecord(unsigned char *data, int &count);
	int CallGetRecord(unsigned char *data, int &count);

	// Displaynachicht senden
	int CallShowMessage(unsigned char *data, int &count);
};

#endif
